import unittest
import warnings


class DecimalDefaultsTest(unittest.TestCase):
    def setUp(self):
        from test_message_field_types_proto import TestDeprecatedField
        warnings.filterwarnings("error")
        self.message = TestDeprecatedField()

    def test_deprecation_warning(self):
        with self.assertRaises(DeprecationWarning):
            x = self.message.old_field

        with self.assertRaises(DeprecationWarning):
            self.message.old_field = 4



import unittest

from pyrobuf.parse_proto import Parser

zero_index_proto = """
message TestZeroIndex {
    required string zero = 0;
}
"""

field_name_collision_proto = """
message TestFieldNameCollision {
    required int32 dupe_name = 1;
    required bytes dupe_name = 2;
}
"""

index_collision_proto = """
message TestIndexCollision {
    required int32 index_one = 1;
    required int64 same_index = 1;
}
"""

nonzero_first_value_proto3 = """
syntax = "proto3";

enum TestNonzeroFirstValue {
    nonzero = 1;
}
"""

enum_name_collision_proto = """
enum TestEnumNamCollision {
    dupe_name = 1;
    dupe_name = 2;
}
"""

value_collision_proto = """
enum TestEValueCollision {
    value_one = 0;
    same_value = 0;
}
"""

global_enum_to_message_collision_proto = """
enum TestEnum {
    dupe_name = 0;
}
message dupe_name {
}
"""

enum_to_field_collision_proto = """
message TestEnumToFieldCollision {
    enum TestEnum {
        dupe_name = 0;
    }
    int32 dupe_name = 1;
}
"""

recursive_message_to_enum_collision_proto = """
message RecursiveMessage {
    message dupe_name {
    }
    enum TestEnum {
        dupe_name = 0;
    }
}
"""

recursive_message_to_field_collision_proto = """
message RecursiveMessage {
    message dupe_name {
    }
    int32 dupe_name = 1;
}
"""



class FieldValidityTest(unittest.TestCase):
    def test_zero_index_field(self):
        with self.assertRaises(AssertionError):
            Parser(zero_index_proto).parse()

    def test_field_name_collision(self):
        with self.assertRaises(AssertionError):
            Parser(field_name_collision_proto).parse()

    def test_field_index_collision(self):
        with self.assertRaises(AssertionError):
            Parser(index_collision_proto).parse()

    def test_enum_nonzero_first_value_in_proto3(self):
        with self.assertRaises(AssertionError):
            Parser(nonzero_first_value_proto3).parse()

    def test_enum_name_collision(self):
        with self.assertRaises(AssertionError):
            Parser(enum_name_collision_proto).parse()

    def test_enum_value_collision(self):
        with self.assertRaises(AssertionError):
            Parser(value_collision_proto).parse()

    def test_global_enum_to_message_collision(self):
        with self.assertRaises(AssertionError):
            Parser(global_enum_to_message_collision_proto).parse()

    def test_enum_to_field_collision(self):
        with self.assertRaises(AssertionError):
            Parser(enum_to_field_collision_proto).parse()

    def test_recursive_message_to_enum_collision(self):
        with self.assertRaises(AssertionError):
            Parser(recursive_message_to_enum_collision_proto).parse()

    def test_recursive_message_to_field_collision(self):
        with self.assertRaises(AssertionError):
            Parser(recursive_message_to_field_collision_proto).parse()


if __name__ == "__main__":
    unittest.main()



import unittest

TestFieldTypes = None


class MessageFieldTypesTest:
    # Mixin with common tests for proto2 and proto3 variants
    def test_bytes_payload_serialize_to_string(self):
        message = TestFieldTypes()
        message.payload = b'\x01\x02\x03\xf0\xf1\xf2'
        self.assertEqual(message.SerializeToString(), b'\n\x06\x01\x02\x03\xf0\xf1\xf2')

    def test_bytes_payload_parse_from_string(self):
        message = TestFieldTypes()
        message.ParseFromString(b'\n\x06\x01\x02\x03\xf0\xf1\xf2')
        self.assertEqual(message.payload, b'\x01\x02\x03\xf0\xf1\xf2')

    def test_uninitialized_message_serialize_to_json(self):
        message = TestFieldTypes()
        self.assertEqual(message.SerializeToJson(), '{}')

    def test_bytes_payload_serialize_to_json(self):
        message = TestFieldTypes()
        message.payload = b'\x01\x02\x03'
        self.assertEqual(message.SerializeToJson(), '{"payload": "AQID"}')

    def test_bytes_payload_parse_from_json(self):
        message = TestFieldTypes()
        message.ParseFromJson('{"payload": "AQID"}')
        self.assertEqual(message.payload, b'\x01\x02\x03')

    def test_list_bytes_serialize_to_string(self):
        message = TestFieldTypes()
        message.list_bytes.append(b'Et')
        message.list_bytes.append(b'tu,')
        message.list_bytes.append(b'Brute?')
        self.assertEqual(message.SerializeToString(), b'*\x02Et*\x03tu,*\x06Brute?')

    def test_list_bytes_parse_from_string(self):
        message = TestFieldTypes()
        message.ParseFromString(b'*\x02Et*\x03tu,*\x06Brute?')
        self.assertEqual(len(message.list_bytes), 3)
        self.assertEqual(message.list_bytes, [b'Et', b'tu,', b'Brute?'])

    def test_list_bytes_serialize_to_string_with_nonascii_chars(self):
        message = TestFieldTypes()
        message.list_bytes.append(b"abc\x93def\x10ghi")
        self.assertEqual(message.SerializePartialToString(), b'*\x0babc\x93def\x10ghi')

    def test_list_bytes_parse_from_string_with_nonascii_chars(self):
        message = TestFieldTypes()
        message.ParseFromString(b'*\x0babc\x93def\x10ghi')
        self.assertEqual(message.list_bytes, [b"abc\x93def\x10ghi"])


class MessageFieldTypes2Test(MessageFieldTypesTest, unittest.TestCase):
    # Test the unique features supported in proto2
    @classmethod
    def setUpClass(cls):
        global TestFieldTypes
        from test_message_field_types_proto import TestFieldTypes

    def test_bytes_payload_with_default_has_default_value(self):
        message = TestFieldTypes()
        self.assertEqual(message.payload_with_default, b'Hello World')

    def test_packed_var_width_list_serialize_to_string(self):
        message = TestFieldTypes()
        message.packed_var_width_list.append(1)
        message.packed_var_width_list.append(-2)
        message.packed_var_width_list.append(3)
        self.assertEqual(message.SerializeToString(), b'\x1a\x0c\x01\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01\x03')

    def test_packed_var_width_list_parse_from_string(self):
        message = TestFieldTypes()
        message.ParseFromString(b'\x1a\x0c\x01\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01\x03')
        self.assertEqual(len(message.packed_var_width_list), 3)
        self.assertEqual(message.packed_var_width_list[0], 1)
        self.assertEqual(message.packed_var_width_list[1], -2)
        self.assertEqual(message.packed_var_width_list[2], 3)

    def test_packed_fixed_width_list_serialize_to_string(self):
        message = TestFieldTypes()
        message.packed_fixed_width_list.append(1)
        message.packed_fixed_width_list.append(-2)
        message.packed_fixed_width_list.append(3)
        self.assertEqual(message.SerializeToString(), b'"\x0c\x01\x00\x00\x00\xfe\xff\xff\xff\x03\x00\x00\x00')

    def test_packed_fixed_width_list_parse_from_string(self):
        message = TestFieldTypes()
        message.ParseFromString(b'"\x0c\x01\x00\x00\x00\xfe\xff\xff\xff\x03\x00\x00\x00')
        self.assertEqual(len(message.packed_fixed_width_list), 3)
        self.assertEqual(message.packed_fixed_width_list[0], 1)
        self.assertEqual(message.packed_fixed_width_list[1], -2)
        self.assertEqual(message.packed_fixed_width_list[2], 3)


class MessageFieldTypes3Test(MessageFieldTypesTest, unittest.TestCase):
    # Some features are no longer supported in proto3; omit those
    @classmethod
    def setUpClass(cls):
        global TestFieldTypes
        from test_message_field_types_3_proto import TestFieldTypes



def test_has_field():
    from test_many_fields_proto import TestManyFields
    test = TestManyFields()

    # Assert HasField false on clean message
    for field in test.Fields():
        assert not test.HasField(field)

    # Set fields
    for idx, field in enumerate(test.Fields()):
        setattr(test, field, idx)

    # Assert HasField true
    for field in test.Fields():
        assert test.HasField(field)

    # Reset and assert HasField false
    test.reset()

    for field in test.Fields():
        assert not test.HasField(field)



def test_message_a():
    from issue_69_proto import A, B
    a = A()
    assert type(a.b.a.b) == B


def test_message_b():
    from issue_69_proto import A, B
    b = B()
    assert type(b.a.b.a) == A



import unittest


GOOGLE_SERIALIZED_MESSAGE = b'\x08\x80\x89\x9a\x81\x02\x10\xc1S\x1a\tgo goats! \x00 d \xc8\x01 ' \
    b'\xac\x02 \x90\x03*A\x08\xb9`\x12\x05hello\x1a\x12\tH\xe1z\x14\xae.\x96@\x12\x07goodbye!\xd7' \
    b'\xa3p=\n/v@*\r\tn\x86\x1b\xf0\xf9!\t@\x12\x02pi2\tsomething2\x17\x08\x80\x89\x9a\x81\x02\x10' \
    b'\xd7\x08\x19\x8d\x97n\x12\x83\xc0\xf3?"\x03fooB\x17\x08\x80\x89\x9a\x81\x02\x10\xd7\x08\x19' \
    b'\x8d\x97n\x12\x83\xc0\xf3?"\x03fooJ7\n\nwhat\'s up?\x12\x0cnothing much\x18\x18 W*\x17\x08' \
    b'\x80\x89\x9a\x81\x02\x10\xd7\x08\x19\x8d\x97n\x12\x83\xc0\xf3?"\x03fooP\xee\x87\xfb\xff\xff' \
    b'\xff\xff\xff\xff\x01X\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01'


class MessageTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global TestTruncated
        from test_truncated_proto import TestTruncated

    def test_deser(self):
        test = TestTruncated()
        parsed = test.ParseFromString(GOOGLE_SERIALIZED_MESSAGE)
        self.assertEqual(parsed, len(GOOGLE_SERIALIZED_MESSAGE))
        self.assertEqual(test.req_field, -80914)
        self.assertEqual(test.negative_32, -1)


if __name__ == "__main__":
    unittest.main()



import unittest


# These can't be imported until the test_imported_enums_proto module has been built.
CLOSE = None
MSG_ONE = None
ExposesInternalEnumConstantsMessage = None
UsesImportedEnumsMessage = None


class ImportedEnumsTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global CLOSE, MSG_ONE, ExposesInternalEnumConstantsMessage, UsesImportedEnumsMessage
        from test_multi_messages_toplevel_enums_proto import MSG_ONE, CLOSE
        from test_imported_enums_proto import UsesImportedEnumsMessage, ExposesInternalEnumConstantsMessage

    def test_message_id_has_default_of_msg_one(self):
        message = UsesImportedEnumsMessage()
        self.assertEqual(message.message_id, MSG_ONE)

    def test_status_has_default_of_close(self):
        message = UsesImportedEnumsMessage()
        self.assertEqual(message.status, CLOSE)

    def test_internal_enum_constants_exposed(self):
        self.assertEqual(ExposesInternalEnumConstantsMessage.INTERNAL, 0)
        self.assertEqual(ExposesInternalEnumConstantsMessage.EXTERNAL, 1)



import unittest

TestRepeatedEnum = None


class RepeatedEnumTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global TestRepeatedEnum
        from test_repeated_enum_proto import TestRepeatedEnum

    def test_repeated_enum_serde(self):
        message1 = TestRepeatedEnum()
        message2 = TestRepeatedEnum()

        message1.list_enum.append(2)
        message1.list_enum.append(0)
        message1.list_enum.append(1)

        buf = message1.SerializeToString()
        message2.ParseFromString(buf)

        for i in range(len(message1.list_enum)):
            self.assertEqual(message1.list_enum[i], message2.list_enum[i])

if __name__ == "__main__":
    unittest.main()



import os
import sys
import time

from create_message import *


HERE = os.path.dirname(os.path.abspath(__file__))
BUILD = os.path.join(HERE, 'build')
lib_path = os.path.join(BUILD, [name for name in os.listdir(BUILD)
                                if name.startswith('lib')].pop())
if lib_path not in sys.path:
    sys.path.insert(0, lib_path)

GOOGLE = True

try:
    import messages.test_message_pb2 as google_test
except ImportError:
    GOOGLE = False

import test_message_proto as an_test


def main():
    if GOOGLE:
        t1 = create_google_test()

    t2 = create_an_test()
    buf = ""

    if GOOGLE:
        start = time.time()
        for i in range(100000):
            buf = t1.SerializeToString()

        end = time.time()
        print("Google took %f seconds to serialize" % (end - start))

    start = time.time()
    for i in range(100000):
        buf = t2.SerializeToString()

    end = time.time()
    print("Pyrobuf took %f seconds to serialize" % (end - start))

    if GOOGLE:
        start = time.time()
        for i in range(100000):
            t1.ParseFromString(buf)

        end = time.time()
        print("Google took %f seconds to deserialize" % (end - start))

    start = time.time()
    for i in range(100000):
        t2.ParseFromString(buf)

    end = time.time()
    print("Pyrobuf took %f seconds to deserialize" % (end - start))

    start = time.time()
    for i in range(100000):
        assert t2.HasField('field')
        assert t2.HasField('string_field')

    end = time.time()
    print("HasField took %f seconds" % (end - start))


if __name__ == "__main__":
    main()



import unittest

Test = None
TestSs1 = None

# Alpha, Beta, Gamma, Delta
GREEK_LETTERS = u"\u0391\u0392\u0393\u0394"


class TestUnicodeStrings(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global Test, TestSs1, TestFieldTypes
        from test_message_proto import Test, TestSs1

    def test_unicode_string_parse_from_string(self):
        message = Test.FromString(b'\x1a\x08\xce\x91\xce\x92\xce\x93\xce\x94')
        self.assertEqual(message.string_field, GREEK_LETTERS)

    def test_unicode_string_parse_from_dict(self):
        message = Test()
        message.ParseFromDict({'string_field': u"\u0391\u0392\u0393\u0394", 'req_field': 1})
        self.assertEqual(message.string_field, GREEK_LETTERS)

    def test_unicode_string_parse_from_json(self):
        message = Test()
        message.ParseFromJson('{"string_field": "\\u0391\\u0392\\u0393\\u0394", "req_field": 1}')
        self.assertEqual(message.string_field, GREEK_LETTERS)

    def test_unicode_string_serialize_to_string(self):
        message = Test()
        message.string_field = GREEK_LETTERS
        self.assertEqual(message.SerializePartialToString(), b'\x1a\x08\xce\x91\xce\x92\xce\x93\xce\x94')

    def test_unicode_string_serialize_to_dict(self):
        message = Test()
        message.string_field = GREEK_LETTERS
        message.req_field = 1
        self.assertEqual(
            message.SerializeToDict(), {'string_field': u"\u0391\u0392\u0393\u0394", 'req_field': 1}
        )

    def test_unicode_string_serialize_to_json(self):
        # Use TestSs1 here because Test has a required field
        message = TestSs1()
        message.field2 = GREEK_LETTERS
        self.assertEqual(
            message.SerializeToJson(), '{"field2": "\\u0391\\u0392\\u0393\\u0394"}'
        )

    def test_repeated_unicode_parse_from_string(self):
        message = TestSs1.FromString(b'2\x04\xce\x91\xce\x922\x04\xce\x93\xce\x94')
        self.assertEqual(message.list_string, [u"\u0391\u0392", u"\u0393\u0394"])

    def test_repeated_unicode_parse_from_dict(self):
        message = TestSs1()
        message.ParseFromDict({'list_string': [u'\u0391\u0392', u'\u0393\u0394']})
        self.assertEqual(message.list_string, [u"\u0391\u0392", u"\u0393\u0394"])

    def test_repeated_unicode_parse_from_json(self):
        message = TestSs1()
        message.ParseFromJson('{"list_string": ["\\u0391\\u0392", "\\u0393\\u0394"]}')
        self.assertEqual(message.list_string, [u"\u0391\u0392", u"\u0393\u0394"])

    def test_repeated_unicode_string_serialize_to_string(self):
        message = TestSs1()
        message.list_string.append(GREEK_LETTERS[:2])  # Alpha, Beta
        message.list_string.append(GREEK_LETTERS[2:])  # Gamma, Delta
        self.assertEqual(message.SerializePartialToString(), b'2\x04\xce\x91\xce\x922\x04\xce\x93\xce\x94')

    def test_repeated_unicode_string_serialize_to_dict(self):
        message = TestSs1()
        message.list_string.append(GREEK_LETTERS[:2])  # Alpha, Beta
        message.list_string.append(GREEK_LETTERS[2:])  # Gamma, Delta
        self.assertEqual(message.SerializeToDict(), {'list_string': [u'\u0391\u0392', u'\u0393\u0394']})

    def test_repeated_unicode_string_serialize_to_json(self):
        message = TestSs1()
        message.list_string.append(GREEK_LETTERS[:2])  # Alpha, Beta
        message.list_string.append(GREEK_LETTERS[2:])  # Gamma, Delta
        self.assertEqual(message.SerializeToJson(), '{"list_string": ["\\u0391\\u0392", "\\u0393\\u0394"]}')



import unittest

from pyrobuf.parse_proto import Parser


syntax2_explicit = '''
syntax = "proto2";

message SearchRequest {
  required string query = 1;
  optional int32 page_number = 2;
  optional int32 result_per_page = 3;
}
'''

syntax2_implicit = '''
message SearchRequest {
  required string query = 1;
  optional int32 page_number = 2;
  optional int32 result_per_page = 3;
}
'''

syntax3_explicit = '''
syntax = "proto3";

message SearchRequest {
  string query = 1;
  int32 page_number = 2;
  int32 result_per_page = 3;
}
'''


class SyntaxTest(unittest.TestCase):

    def test_syntax2(self):
        parser = Parser(syntax2_implicit)
        parser.parse()
        parser = Parser(syntax2_explicit)
        parser.parse()

    def test_syntax3(self):
        parser = Parser(syntax3_explicit)
        parser.parse()



import pytest

from pyrobuf.parse_proto import Parser


map_field_with_enum = """
syntax = "proto3";


enum IPAddressFamily {
  INVALID = 0;
  IPv4 = 1;
  IPv6 = 2;
};

message TenantNetworkPolicy {
  string tenant_name = 1;
  map<int, IPAddressFamily> scopes = 2;
}
"""

map_field_with_message = """
syntax = "proto3";


message ScopeInfo {
  string id = 1;
}

message TenantNetworkPolicy {
  string tenant_name = 1;
  map<string, ScopeInfo> scopes = 2;
}
"""

map_field_with_builtin_types = """
syntax = "proto3";

message TenantNetworkPolicy {
  string tenant_name = 1;
  map<int, string> scopes = 2;
}
"""


class TestMapField(object):

    @staticmethod
    def parsing_result(string):
        return Parser(string).parse()

    def test_map_field_with_message(self):
        result = self.parsing_result(map_field_with_message)
        message = result['messages'][1]

        # assert that all field are listed in the message
        assert [f.name for f in message.fields.values()] == ['tenant_name', 'scopes']

        # assert things about the scope map field
        scopes_field = message.fields[2]
        assert scopes_field.name == 'scopes'
        assert scopes_field.token_type == 'MAP_FIELD'
        assert scopes_field.key_type == 'string'
        # Validate the map value type
        assert scopes_field.type == 'message'
        assert scopes_field.message_name == 'ScopeInfo'

    def test_map_field_with_enum(self):
        result = self.parsing_result(map_field_with_enum)
        message = result['messages'][0]

        # assert that all field are listed in the message
        assert [f.name for f in message.fields.values()] == ['tenant_name', 'scopes']

        # assert things about the ips map field
        scopes_field = message.fields[2]
        assert scopes_field.name == 'scopes'
        assert scopes_field.token_type == 'MAP_FIELD'
        assert scopes_field.key_type == 'int'
        assert scopes_field.type == 'enum'
        assert scopes_field.enum_def == result['enums'][0]

    def test_map_field_with_builtin_types(self):
        result = self.parsing_result(map_field_with_builtin_types)
        message = result['messages'][0]

        # assert that all field are listed in the message
        assert [f.name for f in message.fields.values()] == ['tenant_name', 'scopes']

        # assert things about the ips map field
        scopes_field = message.fields[2]
        assert scopes_field.name == 'scopes'
        assert scopes_field.token_type == 'MAP_FIELD'
        assert scopes_field.key_type == 'int'
        assert scopes_field.type == 'string'




import unittest


TestDecimalDefaultsMessage = None
TestStringDefaultsMessage = None


class DecimalDefaultsTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global TestDecimalDefaultsMessage
        from test_field_defaults_proto import TestDecimalDefaultsMessage

    def setUp(self):
        self.message = TestDecimalDefaultsMessage()

    def test_zero_point_zero(self):
        self.assertEqual(self.message.zero_point_zero, 0.0)

    def test_point_zero(self):
        self.assertEqual(self.message.point_zero, 0.0)

    def test_default_of_zero_point_five(self):
        self.assertEqual(self.message.zero_point_five, 0.5)

    def test_default_of_minus_zero_point_zero(self):
        self.assertEqual(self.message.minus_zero_point_zero, -0.0)

    def test_default_of_point_five(self):
        self.assertEqual(self.message.point_five, 0.5)

    def test_default_of_minus_point_zero(self):
        self.assertEqual(self.message.minus_point_zero, -0.0)

    def test_default_of_minus_point_five(self):
        self.assertEqual(self.message.minus_point_five, -0.5)

    def test_default_of_pi(self):
        self.assertEqual(self.message.pi, 3.141592653589793)

    def test_default_of_one_e_6(self):
        self.assertEqual(self.message.one_e_6, 1e6)

    def test_default_of_one_point_zero_e_6(self):
        self.assertEqual(self.message.one_point_zero_e_6, 1.0e6)

    def test_default_of_one_capital_e_6(self):
        self.assertEqual(self.message.one_capital_e_6, 1E6)

    def test_default_of_minus_one_e_6(self):
        self.assertEqual(self.message.minus_one_e_6, -1e6)

    def test_default_of_one_e_plus_6(self):
        self.assertEqual(self.message.one_e_plus_6, 1e+6)

    def test_default_of_one_e_minus_6(self):
        self.assertEqual(self.message.one_e_minus_6, 1e-6)


class StringDefaultsTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global TestStringDefaultsMessage
        from test_field_defaults_proto import TestStringDefaultsMessage

    def setUp(self):
        self.message = TestStringDefaultsMessage()

    def test_string_message_field_with_default_of_hello_world(self):
        self.assertEqual("Hello World!", self.message.hello)

    def test_string_message_field_with_default_of_empty_string(self):
        self.assertEqual("", self.message.empty)

    def test_string_message_field_with_default_of_three_spaces(self):
        self.assertEqual("   ", self.message.spaces)

    def test_string_message_field_with_default_using_single_quotes(self):
        self.assertEqual("Single quotes", self.message.single_quotes)

    def test_string_message_field_with_default_containing_escaped_chars(self):
        self.assertEqual("Escaped \" quote\n", self.message.escaped_quote)



import unittest


Test = None
TestRef = None
TestSs1 = None
TestSs1Thing = None


class HasFieldTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global Test, TestRef, TestSs1, TestSs1Thing
        from test_message_proto import Test, TestSs1, TestSs1Thing
        from test_ref_message_proto import TestRef

    def test_has_field_for_repeated_field_raises_value_error(self):
        message = Test()
        self.assertRaises(ValueError, message.HasField, 'list_fieldx')

    def test_has_field_for_unset_scalar_field_returns_false(self):
        message = Test()
        self.assertFalse(message.HasField('timestamp'))

    def test_has_field_for_set_scalar_field_returns_true(self):
        message = Test()
        message.timestamp = 123
        self.assertTrue(message.HasField('timestamp'))

    def test_has_field_for_cleared_scalar_field_returns_false(self):
        message = Test()
        message.timestamp = 123
        message.ClearField('timestamp')
        self.assertFalse(message.HasField('timestamp'))

    def test_has_field_for_unset_message_field_returns_false(self):
        message = Test()
        self.assertFalse(message.HasField('substruct'))

    def test_has_field_for_set_message_field_returns_true(self):
        message = Test()
        message.substruct.field2 = "something"
        self.assertTrue(message.HasField('substruct'))

    def test_has_field_for_cleared_message_field_returns_false(self):
        message = Test()
        message.substruct.field2 = "something"
        message.ClearField('substruct')
        self.assertFalse(message.HasField('substruct'))

    def test_has_field_for_scalar_field_of_cleared_message_returns_false(self):
        message = Test()
        message.timestamp = 123
        message.Clear()
        self.assertFalse(message.HasField('timestamp'))

    def test_has_field_for_message_field_of_cleared_message_returns_false(self):
        message = Test()
        message.substruct.field2 = "something"
        message.Clear()
        self.assertFalse(message.HasField('substruct'))

    def test_has_field_for_indirectly_set_message_field_returns_true(self):
        message = Test()
        message.substruct.field3.field1 = 3.14159
        self.assertTrue(message.HasField('substruct'))

    def test_has_field_for_cleared_indirectly_set_message_field_returns_false(self):
        message = Test()
        message.substruct.field3.field1 = 3.14159
        message.ClearField('substruct')
        self.assertFalse(message.HasField('substruct'))

    def test_has_field_for_message_field_set_by_repeated_field_returns_true(self):
        message = Test()
        message.substruct.list.append(3)
        self.assertTrue(message.HasField('substruct'))

    def test_has_field_for_message_field_set_by_repeated_message_field_returns_true(self):
        message = Test()
        message.substruct.list_object.append(TestSs1Thing())
        self.assertTrue(message.HasField('substruct'))

    def test_has_field_for_string_field_set_by_deserialization_returns_true(self):
        ref_message = TestRef()
        ref_message.field3 = "Hello World"
        ref_message2 = TestRef.FromString(ref_message.SerializeToString())
        self.assertTrue(ref_message2.HasField('field3'))

    def test_has_field_for_scalar_field_set_by_deserialization_returns_true(self):
        ref_message = TestRef()
        ref_message.field1 = 1
        ref_message2 = TestRef.FromString(ref_message.SerializeToString())
        self.assertTrue(ref_message2.HasField('field1'))

    def test_has_field_for_message_field_set_by_deserialization_returns_true(self):
        ss1_message = TestSs1()
        ss1_message.field3.field1 = 3.14159
        ss1_message2 = TestSs1.FromString(ss1_message.SerializeToString())
        self.assertTrue(ss1_message2.field3._is_present_in_parent)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: test_ref_message.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='test_ref_message.proto',
  package='',
  serialized_pb=_b('\n\x16test_ref_message.proto\"L\n\x07TestRef\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x0e\n\x06\x66ield1\x18\x02 \x01(\x03\x12\x0e\n\x06\x66ield2\x18\x03 \x01(\x01\x12\x0e\n\x06\x66ield3\x18\x04 \x01(\t')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TESTREF = _descriptor.Descriptor(
  name='TestRef',
  full_name='TestRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='TestRef.timestamp', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field1', full_name='TestRef.field1', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field2', full_name='TestRef.field2', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field3', full_name='TestRef.field3', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=102,
)

DESCRIPTOR.message_types_by_name['TestRef'] = _TESTREF

TestRef = _reflection.GeneratedProtocolMessageType('TestRef', (_message.Message,), dict(
  DESCRIPTOR = _TESTREF,
  __module__ = 'test_ref_message_pb2'
  # @@protoc_insertion_point(class_scope:TestRef)
  ))
_sym_db.RegisterMessage(TestRef)


# @@protoc_insertion_point(module_scope)



import random


def gen_rand_int32():
    return random.randint(-2**31, 2**31-1)


def gen_rand_uint32():
    return random.randint(0, 2**32-1)


def gen_rand_int64():
    return random.randint(-2**63, 2**63-1)


def gen_rand_uint64():
    return random.randint(0, 2**64-1)


def test_int32():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_int32()
    a.Int32 = v
    assert Int.FromString(a.SerializeToString()).Int32 == v


def test_int64():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_int64()
    a.Int64 = v
    assert Int.FromString(a.SerializeToString()).Int64 == v


def test_uint32():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_uint32()
    a.UInt32 = v
    assert Int.FromString(a.SerializeToString()).UInt32 == v


def test_uint64():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_uint64()
    a.UInt64 = v
    assert Int.FromString(a.SerializeToString()).UInt64 == v


def test_sint32():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_int32()
    a.SInt32 = v
    assert Int.FromString(a.SerializeToString()).SInt32 == v


def test_sint64():
    from test_signed_integer_proto import Int
    a = Int()
    v = gen_rand_int64()
    a.SInt64 = v
    assert Int.FromString(a.SerializeToString()).SInt64 == v



import time
from array import array

from pyrobuf_list import *

TEST_ITERS = 1000000


def main():
    start = time.time()
    x = []
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("list took %f seconds" % (end - start))

    start = time.time()
    x = TypedList(float)
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("TypedList took %f seconds" % (end - start))

    start = time.time()
    x = DoubleList()
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("DoubleList took %f seconds" % (end - start))

    start = time.time()
    x = FloatList()
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("FloatList took %f seconds" % (end - start))

    start = time.time()
    x = array('f')
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("array 'f' took %f seconds" % (end - start))

    start = time.time()
    x = array('d')
    y = 0

    for i in range(TEST_ITERS):
        x.append(float(i))

    for i in x:
        y += i

    end = time.time()
    print("array 'd' took %f seconds" % (end - start))

    start = time.time()
    x = []
    y = 0

    for i in range(TEST_ITERS):
        x.append(i)

    for i in x:
        y += i

    end = time.time()
    print("list took %f seconds" % (end - start))

    start = time.time()
    x = IntList()
    y = 0

    for i in range(TEST_ITERS):
        x.append(i)

    for i in x:
        y += i

    end = time.time()
    print("IntList took %f seconds" % (end - start))

    start = time.time()
    x = Int32List()
    y = 0

    for i in range(TEST_ITERS):
        x.append(i)

    for i in x:
        y += i

    end = time.time()
    print("Int32List took %f seconds" % (end - start))

    start = time.time()
    x = array('i')
    y = 0

    for i in range(TEST_ITERS):
        x.append(i)

    for i in x:
        y += i

    end = time.time()
    print("array 'i' took %f seconds" % (end - start))


if __name__ == "__main__":
    main()



import unittest

from pyrobuf_list import BytesList, StringList


NON_STRING_VALUE = 1


class TestBytesList(unittest.TestCase):
    def test_append(self):
        bytes_list = BytesList()
        bytes_list.append(b"some string")
        self.assertEqual(bytes_list, [b"some string"])

    def test_append_with_non_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.append(NON_STRING_VALUE)

    def test_append_with_unicode_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.append(u"some string")

    def test_delitem(self):
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        del bytes_list[2]
        self.assertEqual(bytes_list, [b"zero", b"one", b"three"])

    def test_delitem_with_slice(self):
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        del bytes_list[1:3]
        self.assertEqual(bytes_list, [b"zero", b"three"])

    def test_extend(self):
        bytes_list = BytesList()
        bytes_list.extend([b"some string", b"another string"])
        self.assertEqual(bytes_list, [b"some string", b"another string"])

    def test_extend_with_non_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.extend([b"some string", NON_STRING_VALUE])

    def test_extend_with_unicode_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.extend([u"some string"])

    def test_extend_with_empty_sequence(self):
        bytes_list = BytesList()
        bytes_list.extend([])
        self.assertEqual(bytes_list, [])

    def test_insert(self):
        bytes_list = BytesList()
        bytes_list.insert(0, b"some string")
        self.assertEqual(bytes_list, [b"some string"])

    def test_insert_with_non_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.insert(0, NON_STRING_VALUE)

    def test_insert_with_byte_string(self):
        bytes_list = BytesList()
        with self.assertRaises(TypeError):
            bytes_list.insert(0, u"some string")

    def test_setitem_using_integer(self):
        bytes_list = BytesList()
        bytes_list.append(b"some string")
        bytes_list[0] = b"another string"
        self.assertEqual(bytes_list[0], b"another string")

    def test_setitem_using_slice(self):
        # This will call BytesList.__setslice__ on Python 2
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        bytes_list[1:3] = [b"four", b"five"]
        self.assertEqual(bytes_list, [b"zero", b"four", b"five", b"three"])

    def test_setitem_using_slice_with_non_string(self):
        # This will call BytesList.__setslice__ on Python 2
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        with self.assertRaises(TypeError):
            bytes_list[1:3] = [u"four", NON_STRING_VALUE]

    def test_setitem_using_slice_with_empty_sequence(self):
        # This will call BytesList.__setslice__ on Python 2
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        bytes_list[1:3] = []
        self.assertEqual(bytes_list, [b"zero", b"three"])

    def test_setitem_using_stepped_slice(self):
        # This will call BytesList.__setitem__ on both Python 2 and Python 3
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        bytes_list[0:3:2] = [b"four", b"five"]
        self.assertEqual(bytes_list, [b"four", b"one", b"five", b"three"])

    def test_setitem_using_stepped_slice_with_non_string(self):
        # This will call BytesList.__setitem__ on both Python 2 and Python 3
        bytes_list = BytesList()
        bytes_list.extend([b"zero", b"one", b"two", b"three"])
        with self.assertRaises(TypeError):
            bytes_list[0:3:2] = [b"four", NON_STRING_VALUE]


class TestStringList(unittest.TestCase):
    def test_append(self):
        str_list = StringList()
        str_list.append(u"some string")
        self.assertEqual(str_list, [u"some string"])

    def test_append_with_non_string(self):
        str_list = StringList()
        with self.assertRaises(TypeError):
            str_list.append(NON_STRING_VALUE)

    def test_append_with_byte_string(self):
        str_list = StringList()
        str_list.append(b"some string")
        self.assertEqual(str_list[0], b"some string".decode('utf-8'))

    def test_delitem(self):
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        del str_list[2]
        self.assertEqual(str_list, [u"zero", u"one", u"three"])

    def test_delitem_with_slice(self):
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        del str_list[1:3]
        self.assertEqual(str_list, [u"zero", u"three"])

    def test_extend(self):
        str_list = StringList()
        str_list.extend([u"some string", u"another string"])
        self.assertEqual(str_list, [u"some string", u"another string"])

    def test_extend_with_non_string(self):
        str_list = StringList()
        with self.assertRaises(TypeError):
            str_list.extend([u"some string", NON_STRING_VALUE])

    def test_extend_with_byte_string(self):
        str_list = StringList()
        str_list.extend([b"some string"])
        self.assertEqual(str_list[0], b"some string".decode('utf-8'))

    def test_extend_with_empty_sequence(self):
        str_list = StringList()
        str_list.extend([])
        self.assertEqual(str_list, [])

    def test_insert(self):
        str_list = StringList()
        str_list.insert(0, u"some string")
        self.assertEqual(str_list, [u"some string"])

    def test_insert_with_non_string(self):
        str_list = StringList()
        with self.assertRaises(TypeError):
            str_list.insert(0, NON_STRING_VALUE)

    def test_insert_with_byte_string(self):
        str_list = StringList()
        str_list.insert(0, b"some string")
        self.assertEqual(str_list[0], b"some string".decode('utf-8'))

    def test_setitem_using_integer(self):
        str_list = StringList()
        str_list.append(u"some string")
        str_list[0] = u"another string"
        self.assertEqual(str_list[0], u"another string")

    def test_setitem_using_slice(self):
        # This will call StringList.__setslice__ on Python 2
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        str_list[1:3] = [u"four", u"five"]
        self.assertEqual(str_list, [u"zero", u"four", u"five", u"three"])

    def test_setitem_using_slice_with_non_string(self):
        # This will call StringList.__setslice__ on Python 2
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        with self.assertRaises(TypeError):
            str_list[1:3] = [u"four", NON_STRING_VALUE]

    def test_setitem_using_slice_with_empty_sequence(self):
        # This will call StringList.__setslice__ on Python 2
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        str_list[1:3] = []
        self.assertEqual(str_list, [u"zero", u"three"])

    def test_setitem_using_stepped_slice(self):
        # This will call StringList.__setitem__ on both Python 2 and Python 3
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        str_list[0:3:2] = [u"four", u"five"]
        self.assertEqual(str_list, [u"four", u"one", u"five", u"three"])

    def test_setitem_using_stepped_slice_with_non_string(self):
        # This will call StringList.__setitem__ on both Python 2 and Python 3
        str_list = StringList()
        str_list.extend([u"zero", u"one", u"two", u"three"])
        with self.assertRaises(TypeError):
            str_list[0:3:2] = [u"four", NON_STRING_VALUE]



import unittest

Test = None
TestSs1 = None
TestSs3 = None


class MergeFromTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global Test, TestSs1, TestSs3
        from test_message_proto import Test, TestSs1, TestSs3

    def test_merge_from_wrong_type_raises_type_error(self):
        dest = Test()
        self.assertRaises(TypeError, dest.MergeFrom, 1)

    def test_merge_from_empty_message_notifies_parent(self):
        source = TestSs1()
        dest = Test()
        dest.substruct.MergeFrom(source)
        self.assertTrue(dest.HasField('substruct'))

    def test_merge_from_empty_message_does_not_set_scalar_field(self):
        source = TestSs3()
        dest = TestSs3()
        dest.MergeFrom(source)
        self.assertFalse(dest.HasField('int_field'))

    def test_merge_from_empty_message_does_not_set_enum_field(self):
        source = Test()
        dest = Test()
        dest.MergeFrom(source)
        self.assertFalse(dest.HasField('enum_field'))

    def test_merge_from_empty_message_does_not_set_message_field(self):
        source = TestSs3()
        dest = TestSs3()
        dest.MergeFrom(source)
        self.assertFalse(dest.HasField('substruct_ref'))

    def test_merge_from_does_set_scalar_field_that_is_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.int_field = 3
        dest.MergeFrom(source)
        self.assertEqual(dest.int_field, 3)

    def test_merge_from_does_set_enum_field_that_is_set_in_source(self):
        source = Test()
        dest = Test()
        source.enum_field = Test.TEST_ENUM_FIELD_2
        dest.MergeFrom(source)
        self.assertEqual(dest.enum_field, Test.TEST_ENUM_FIELD_2)

    def test_merge_from_does_merge_message_field_that_is_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.substruct_ref.timestamp = 123
        dest.MergeFrom(source)
        self.assertEqual(dest.substruct_ref.timestamp, 123)

    def test_merge_from_does_not_set_field_that_is_not_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.int_field = 3
        dest.MergeFrom(source)
        self.assertFalse(dest.HasField('another_int_field'))

    def test_merge_from_does_not_modify_field_that_is_not_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.int_field = 3
        dest.another_int_field = 5
        dest.MergeFrom(source)
        self.assertEqual(dest.another_int_field, 5)

    def test_merge_from_overwrites_scalar_field_that_is_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.int_field = 5
        dest.int_field = 3
        dest.MergeFrom(source)
        self.assertEqual(dest.int_field, 5)

    def test_merge_from_overwrites_message_field_that_is_set_in_source(self):
        source = TestSs3()
        dest = TestSs3()
        source.substruct_ref.timestamp = 123
        dest.substruct_ref.timestamp = 456
        dest.MergeFrom(source)
        self.assertEqual(dest.substruct_ref.timestamp, 123)

    def test_merge_from_sets_repeated_scalar_field_that_is_set_in_source(self):
        source = Test()
        dest = Test()
        source.list_fieldx.append(3)
        dest.MergeFrom(source)
        self.assertEqual(len(dest.list_fieldx), 1)
        self.assertEqual(dest.list_fieldx[0], 3)

    def test_merge_from_extends_repeated_scalar_field_that_is_set_in_source(self):
        source = Test()
        dest = Test()
        source.list_fieldx.append(3)
        dest.list_fieldx.append(4)
        dest.MergeFrom(source)
        self.assertEqual(len(dest.list_fieldx), 2)
        self.assertEqual(dest.list_fieldx[0], 4)
        self.assertEqual(dest.list_fieldx[1], 3)

    def test_merge_from_sets_repeated_message_field_that_is_set_in_source(self):
        source = Test()
        dest = Test()
        source.list_ref.add().field1 = 3
        dest.MergeFrom(source)
        self.assertEqual(len(dest.list_ref), 1)
        self.assertEqual(dest.list_ref[0].field1, 3)

    def test_merge_from_extends_repeated_message_field_that_is_set_in_source(self):
        source = Test()
        dest = Test()
        source.list_ref.add().field1 = 3
        dest.list_ref.add().field1 = 4
        dest.MergeFrom(source)
        self.assertEqual(len(dest.list_ref), 2)
        self.assertEqual(dest.list_ref[0].field1, 4)
        self.assertEqual(dest.list_ref[1].field1, 3)



import sys
import unittest

from create_message import *


GOOGLE_SERIALIZED_MESSAGE = b'\x08\x80\x89\x9a\x81\x02\x10\xc1S\x1a\tgo goats! \x00 d \xc8\x01 ' \
    b'\xac\x02 \x90\x03*A\x08\xb9`\x12\x05hello\x1a\x12\tH\xe1z\x14\xae.\x96@\x12\x07goodbye!\xd7' \
    b'\xa3p=\n/v@*\r\tn\x86\x1b\xf0\xf9!\t@\x12\x02pi2\tsomething2\x17\x08\x80\x89\x9a\x81\x02\x10' \
    b'\xd7\x08\x19\x8d\x97n\x12\x83\xc0\xf3?"\x03fooB\x17\x08\x80\x89\x9a\x81\x02\x10\xd7\x08\x19' \
    b'\x8d\x97n\x12\x83\xc0\xf3?"\x03fooJ7\n\nwhat\'s up?\x12\x0cnothing much\x18\x18 W*\x17\x08' \
    b'\x80\x89\x9a\x81\x02\x10\xd7\x08\x19\x8d\x97n\x12\x83\xc0\xf3?"\x03fooP\xee\x87\xfb\xff\xff' \
    b'\xff\xff\xff\xff\x01X\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01'


class MessageTest(unittest.TestCase):
    def create_msg(self):
        return create_an_test(protover=2)

    def test_ser_deser(self):
        if sys.version_info.major == 2:
            t1 = create_google_test()
            buf1 = t1.SerializeToString()
        else:
            buf1 = GOOGLE_SERIALIZED_MESSAGE

        t2 = self.create_msg()
        buf2 = t2.SerializeToString()

        self.assertEqual(buf1, buf2)

        if sys.version_info.major == 2:
            t1.ParseFromString(buf1)
            buf1 = t1.SerializeToString()
        else:
            buf1 = GOOGLE_SERIALIZED_MESSAGE

        t2.ParseFromString(buf1)
        buf2 = t2.SerializeToString()

        self.assertEqual(buf1, buf2)

    def test_json(self):
        test = self.create_msg()
        json = test.SerializeToJson()

        test.ParseFromJson(json)
        self.assertEqual(json, test.SerializeToJson())

    def test_dict(self):
        test = self.create_msg()
        the_dict = test.SerializeToDict()

        test.ParseFromDict(the_dict)
        self.assertEqual(the_dict, test.SerializeToDict())


class Message3Test(MessageTest):
    def create_msg(self):
        return create_an_test(protover=3)


if __name__ == "__main__":
    unittest.main()



import unittest

TestBool = None


class BoolFieldTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global TestBool
        from test_bool_proto import TestBool

    def test_bool_field(self):
        message = TestBool()
        self.assertIsInstance(message.bool_field, bool)
        self.assertIs(message.bool_field, False)
        message.bool_field = True
        self.assertIs(message.bool_field, True)

    def test_bool_list(self):
        message = TestBool()
        message.bool_list.append(False)
        self.assertIsInstance(message.bool_list[0], bool)
        self.assertIs(message.bool_list[0], False)
        message.bool_list.append(True)
        self.assertIs(message.bool_list[1], True)

    def test_serde(self):
        t1 = TestBool()
        t2 = TestBool()
        buf1 = t1.SerializeToString()
        t2.ParseFromString(buf1)
        buf2 = t2.SerializeToString()
        self.assertEqual(buf1, buf2)



import unittest

from pyrobuf.parse_proto import Parser


class TestMissingLBraces(unittest.TestCase):
    def test_enum_with_missing_lbrace_raises_exception(self):
        s = "enum MissingLBrace    NO = 0;    YES = 1;}"
        parser = Parser(s)

        self.assertRaises(Exception, parser.parse)

    def test_message_with_missing_lbrace_raises_exception(self):
        s = "message MissingLBrace    required int32 i = 1;    optional int32 j = 2;}"
        parser = Parser(s)

        self.assertRaises(Exception, parser.parse)



import unittest

Test = None
TestSs1 = None
TestSs3 = None


class MergeFromTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global Test, TestSs1, TestSs3
        from test_message_proto import Test, TestSs1, TestSs3

    def test_message_init_with_bad_keyword_arg_raises_value_error(self):
        self.assertRaises(ValueError, Test, non_existant=3)

    def test_message_init_with_scalar_field_keyword_arg_sets_value(self):
        message = Test(req_field=3)
        self.assertEqual(message.req_field, 3)

    def test_message_init_with_message_field_keyword_arg_merges(self):
        ss3_message = TestSs3()
        ss3_message.int_field = 7
        message = Test(another_substruct=ss3_message)
        self.assertFalse(message.another_substruct is ss3_message)
        self.assertEqual(message.another_substruct.int_field, 7)

    def test_message_init_with_repeated_scalar_field_keyword_arg_extends(self):
        source_message = Test()
        source_message.list_fieldx.append(7)
        source_message.list_fieldx.append(8)
        source_message.list_fieldx.append(9)
        message = Test(list_fieldx=source_message.list_fieldx)
        self.assertEqual(list(message.list_fieldx), [7, 8, 9])

    def test_message_init_with_repeated_message_field_keyword_arg_extends(self):
        source_message = Test()
        source_message.list_ref.add().timestamp = 123
        source_message.list_ref.add().timestamp = 456
        message = Test(list_ref=source_message.list_ref)
        self.assertEqual(message.list_ref[0].timestamp, 123)
        self.assertEqual(message.list_ref[1].timestamp, 456)

    def test_message_init_by_repeated_field_add_method_passes_keyword_args(self):
        message = Test()
        message.list_ref.add(timestamp=123, field1=456)
        self.assertEqual(message.list_ref[0].timestamp, 123)
        self.assertEqual(message.list_ref[0].field1, 456)



import unittest

from pyrobuf_list import *


class DoubleListTest(unittest.TestCase):

    def test_append_get_set_len(self):
        x = DoubleList()

        for i in range(40):
            x.append(i)

        for i in range(40):
            self.assertEqual(x[i], i)
            x[i] *= 2

        for i in range(40):
            self.assertEqual(x[i], 2 * i)

        with self.assertRaises(IndexError):
            x[40]

        with self.assertRaises(IndexError):
            x[-41] = 1

        self.assertEqual(x[-1], x[39])
        self.assertEqual(x[-5], x[35])

        self.assertEqual(len(x), 40)

    def test_extend(self):
        x1 = DoubleList()
        x2 = DoubleList()

        for i in range(5):
            x1.append(i)
            x2.append(5 + i)

        x1.extend(x2)

        for i in range(10):
            self.assertEqual(x1[i], i)

    def test_insert(self):
        x = DoubleList()

        for i in range(16):
            x.append(i)

        for i in range(16):
            x.insert(8 + i, 10 * i)

        for i in range(16):
            self.assertEqual(x[8 + i], 10 * i)

    def test_pop(self):
        x = DoubleList()

        for i in range(5):
            x.append(i)

        for i in range(5):
            a = x.pop()
            self.assertEqual(a, 5 - i - 1)
            self.assertEqual(len(x), 5 - i - 1)

        with self.assertRaises(IndexError):
            x.pop()

    def test_remove(self):
        x = DoubleList()

        for i in range(5):
            x.append(i)

        x.remove(3)
        self.assertEqual(x[3], 4)

        with self.assertRaises(ValueError):
            x.remove(7)

    def test_extend_with_generic_list(self):
        x1 = DoubleList()
        x2 = [1.0, 2.0, 3.0, 4.0, 5.0]

        x1.extend(x2)

        for i in range(5):
            self.assertEqual(x1[i], x2[i])


if __name__ == "__main__":
    unittest.main()



import pyrobuf_util


def test_varint_encode_0():
    assert pyrobuf_util.to_varint(0) == b'\x00'


def test_varint_encode_1():
    assert pyrobuf_util.to_varint(1) == b'\x01'


def test_varint_encode_12345():
    assert pyrobuf_util.to_varint(12345) == b'\xb9`'


def test_varint_encode_max_int32():
    assert pyrobuf_util.to_varint(2**31-1) == b'\xff\xff\xff\xff\x07'


def test_varint_encode_max_int64():
    assert pyrobuf_util.to_varint(2**63-1) == b'\xff\xff\xff\xff\xff\xff\xff\xff\x7f'


def test_varint_encode_negative_1():
    assert pyrobuf_util.to_varint(-1) == b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01'


def test_varint_encode_negative_12345():
    assert pyrobuf_util.to_varint(-12345) == b'\xc7\x9f\xff\xff\xff\xff\xff\xff\xff\x01'


def test_varint_encode_negative_max_int32():
    assert pyrobuf_util.to_varint(-2**31) == b'\x80\x80\x80\x80\xf8\xff\xff\xff\xff\x01'


def test_varint_encode_negative_max_int64():
    assert pyrobuf_util.to_varint(-2**63) == b'\x80\x80\x80\x80\x80\x80\x80\x80\x80\x01'


def test_varint_decode_0():
    assert pyrobuf_util.from_varint(b'\x00') == (0, 1)


def test_varint_decode_1():
    assert pyrobuf_util.from_varint(b'\x01') == (1, 1)


def test_varint_decode_12345():
    assert pyrobuf_util.from_varint(b'\xb9`') == (12345, 2)


def test_varint_decode_max_int32():
    assert pyrobuf_util.from_varint(b'\xff\xff\xff\xff\x07') == (2**31-1, 5)


def test_varint_decode_max_int64():
    assert pyrobuf_util.from_varint(b'\xff\xff\xff\xff\xff\xff\xff\xff\x7f') == (2**63-1, 9)


def test_varint_decode_negative_1():
    assert pyrobuf_util.from_varint(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01') == (-1, 10)


def test_varint_decode_negative_12345():
    assert pyrobuf_util.from_varint(b'\xc7\x9f\xff\xff\xff\xff\xff\xff\xff\x01') == (-12345, 10)


def test_varint_decode_negative_max_int32():
    assert pyrobuf_util.from_varint(b'\x80\x80\x80\x80\xf8\xff\xff\xff\xff\x01') == (-2**31, 10)


def test_varint_decode_negative_max_int64():
    assert pyrobuf_util.from_varint(b'\x80\x80\x80\x80\x80\x80\x80\x80\x80\x01') == (-2**63, 10)


# Now for signed varints

def test_signed_varint_encode_0():
    assert pyrobuf_util.to_signed_varint(0) == b'\x00'


def test_signed_varint_encode_1():
    assert pyrobuf_util.to_signed_varint(1) == b'\x02'


def test_signed_varint_encode_12345():
    assert pyrobuf_util.to_signed_varint(12345) == b'\xf2\xc0\x01'


def test_signed_varint_encode_max_int32():
    assert pyrobuf_util.to_signed_varint(2**31-1) == b'\xfe\xff\xff\xff\x0f'


def test_signed_varint_encode_max_int64():
    assert pyrobuf_util.to_signed_varint(2**63-1) == b'\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01'


def test_signed_varint_encode_negative_1():
    assert pyrobuf_util.to_signed_varint(-1) == b'\x01'


def test_signed_varint_encode_negative_12345():
    assert pyrobuf_util.to_signed_varint(-12345) == b'\xf1\xc0\x01'


def test_signed_varint_encode_negative_max_int32():
    assert pyrobuf_util.to_signed_varint(-2**31) == b'\xff\xff\xff\xff\x0f'


def test_signed_varint_encode_negative_max_int64():
    assert pyrobuf_util.to_signed_varint(-2**63) == b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01'


def test_signed_varint_decode_0():
    assert pyrobuf_util.from_signed_varint(b'\x00') == (0, 1)


def test_signed_varint_decode_1():
    assert pyrobuf_util.from_signed_varint(b'\x02') == (1, 1)


def test_signed_varint_decode_12345():
    assert pyrobuf_util.from_signed_varint(b'\xf2\xc0\x01') == (12345, 3)


def test_signed_varint_decode_max_int32():
    assert pyrobuf_util.from_signed_varint(b'\xfe\xff\xff\xff\x0f') == (2**31-1, 5)


def test_signed_varint_decode_max_int64():
    assert pyrobuf_util.from_signed_varint(b'\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01') == (2**63-1, 10)


def test_signed_varint_decode_negative_1():
    assert pyrobuf_util.from_signed_varint(b'\x01') == (-1, 1)


def test_signed_varint_decode_negative_12345():
    assert pyrobuf_util.from_signed_varint(b'\xf1\xc0\x01') == (-12345, 3)


def test_signed_varint_decode_negative_max_int32():
    assert pyrobuf_util.from_signed_varint(b'\xff\xff\xff\xff\x0f') == (-2**31, 5)


def test_signed_varint_decode_negative_max_int64():
    assert pyrobuf_util.from_signed_varint(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01') == (-2**63, 10)



from pyrobuf.parse_proto import Parser

proto_def = """
syntax = "proto3";

message LBService {
  // Virtual IP used by clients to reach the service behind load balancers.
  string vip = 1;

  oneof ServiceName {
    // URL used by clients to reach service behind load balancers.
    string url = 2;

    // Service name if a non-standard mechanism is used for service discovery.
    string name = 3;
  }
}
"""


class TestOneOf(object):

    def test_oneof(self):
        result = Parser(proto_def).parse()
        message = result['messages'][0]

        # assert that all field are listed in the message
        assert [f.name for f in message.fields.values()] == ['vip', 'url', 'name']

        # assert that the oneofs list their child field names
        assert len(message.oneofs) == 1
        assert 'ServiceName' in message.oneofs
        assert message.oneofs['ServiceName'].fields == ['url', 'name']




import os
import re


class Parser(object):

    syntax = 2

    # Tokens ordered roughly by priority
    tokens = [
        ('COMMENT_OL', r'\/\/.*?\n'),
        ('COMMENT_ML', r'\/\*(?:.|[\r\n])*?\*\/'),
        ('OPTION', r'option\s+((?:.|[\n\r])*?);'),
        ('IMPORT', r'import\s+"(.+?).proto"\s*;'),
        ('MESSAGE', r'message\s+([A-Za-z_][0-9A-Za-z_]*)'),
        ('ENUM', r'enum\s+([A-Za-z_][0-9A-Za-z_]*)'),
        ('PACKAGE', r'package\s+[A-Za-z_][0-9A-Za-z_\.]*\s*;'),
        ('SYNTAX', r'syntax\s*=\s*"(.*?)"\s*;'),
        ('EXTEND', r'extend\s+([A-Za-z_][0-9A-Za-z_\.]*)'),
        ('EXTENSION', r'extensions\s+(\d+)\s+to\s+(\d+|max)\s*;'),
        ('ONEOF', r'oneof\s+([A-Za-z_][0-9A-Za-z_]*)'),
        ('MODIFIER', r'(optional|required|repeated)'),
        ('FIELD', r'([A-Za-z][0-9A-Za-z_]*)\s+([A-Za-z][0-9A-Za-z_]*)\s*=\s*(\d+)'),
        ('MAP_FIELD', r'map<([A-Za-z][0-9A-Za-z_]+),\s*([A-Za-z][0-9A-Za-z_]+)>\s+([A-Za-z][0-9A-Za-z_]*)\s*=\s*(\d+)'),
        ('DEFAULT', r'default\s*='),
        ('PACKED', r'packed\s*=\s*(true|false)'),
        ('DEPRECATED', r'deprecated\s*=\s*(true|false)'),
        ('CUSTOM', r'(\([A-Za-z][0-9A-Za-z_]*\).[A-Za-z][0-9A-Za-z_]*)\s*='),
        ('LBRACKET', r'\['),
        ('RBRACKET', r'\]\s*;'),
        ('LBRACE', r'\{'),
        ('RBRACE', r'\}\s*;{0,1}'),
        ('COMMA', r','),
        ('SKIP', r'\s'),
        ('SEMICOLON', r';'),
        ('NUMERIC', r'(-?[0-9]*\.?[0-9]+(?:[eE][-+]?[0-9]+)?)'),
        ('STRING', r'("(?:\\.|[^"\\])*"|\'(?:\\.|[^"\\])*\')'),
        ('BOOLEAN', r'(true|false)'),
        ('ENUM_FIELD_WITH_VALUE', r'([A-Za-z_][0-9A-Za-z_]*)\s*=\s*(0x[0-9A-Fa-f]+|-\d+|\d+)'),
        ('ENUM_FIELD', r'([A-Za-z_][0-9A-Za-z_]*)')
    ]

    parsable_tokens = (
        'OPTION',
        'SYNTAX',
        'IMPORT',
        'MESSAGE',
        'MODIFIER',
        'FIELD',
        'MAP_FIELD',
        'LBRACKET',
        'RBRACKET',
        'DEFAULT',
        'PACKED',
        'DEPRECATED',
        'COMMA',
        'SEMICOLON',
        'ENUM',
        'LBRACE',
        'RBRACE',
        'EXTENSION',
        'ONEOF',
        'EXTEND',
    )

    # These tokens are parsed by the parser but are not supported by the
    # code that build the C extension files and definitions
    unsupported_tokens = ('MAP_FIELD', 'ONEOF',)

    scalars = {
        'double', 'float', 'int32', 'int64', 'uint32', 'uint64', 'sint32', 'sint64',
        'fixed32', 'fixed64', 'sfixed32', 'sfixed64', 'bool', 'enum', 'timestamp'
    }

    floats = {'double', 'float'}

    list_type_map = {
        'float':    'FloatList',
        'double':   'DoubleList',
        'enum':     'Int32List',
        'int32':    'Int32List',
        'sint32':   'Int32List',
        'sfixed32': 'Int32List',
        'uint32':   'Uint32List',
        'fixed32':  'Uint32List',
        'bool':     'BintList',
        'int64':    'Int64List',
        'sint64':   'Int64List',
        'sfixed64': 'Int64List',
        'uint64':   'Uint64List',
        'fixed64':  'Uint64List',
        'string':   'StringList',
        'bytes':    'BytesList'
    }

    c_type_map = {
        'float':    'float',
        'double':   'double',
        'int32':    'int32_t',
        'sint32':   'int32_t',
        'sfixed32': 'int32_t',
        'uint32':   'uint32_t',
        'fixed32':  'uint32_t',
        'int64':    'int64_t',
        'sint64':   'int64_t',
        'sfixed64': 'int64_t',
        'uint64':   'uint64_t',
        'fixed64':  'uint64_t',
        'bool':     'uint32_t',
        'enum':     'int32_t',
        'timestamp': 'uint32_t'
    }

    getter_map = {
        'bool':     'get_varint32',
        'enum':     'get_varint32',
        'int32':    'get_varint32',
        'sint32':   'get_signed_varint32',
        'uint32':   'get_varint32',
        'int64':    'get_varint64',
        'sint64':   'get_signed_varint64',
        'uint64':   'get_varint64'
    }

    setter_map = {
        'bool':     'set_varint32',
        'enum':     'set_varint32',
        'int32':    'set_varint32',
        'sint32':   'set_signed_varint32',
        'uint32':   'set_varint32',
        'int64':    'set_varint64',
        'sint64':   'set_signed_varint64',
        'uint64':   'set_varint64'
    }

    token_regex = '|'.join('(?P<%s>%s)' % pair for pair in tokens)
    get_token = re.compile(token_regex).match
    token_getter = {key: re.compile(val).match for key, val in tokens}

    def __init__(self, string):
        self.string = string
        self.lines = string.split('\n')

    def tokenize(self, disabled_token_types):
        token_type_to_token_class = self.get_token_type_to_token_class_map()

        pos = 0
        line = 0

        m = self.get_token(self.string, pos)
        while m is not None:
            token_type = m.lastgroup
            subm = self.token_getter[token_type](m.group(token_type))
            vals = subm.groups()

            assert token_type not in disabled_token_types, "Disabled token '{}' found on line {}: {}".format(
                token_type, line + 1, self.lines[line])

            # ENUM_FIELD_WITH_VALUE has different regex
            # but same class as ENUM_FIELD
            if token_type == 'ENUM_FIELD_WITH_VALUE':
                token_type = 'ENUM_FIELD'

            if token_type in token_type_to_token_class:
                token_class = token_type_to_token_class[token_type]
                yield token_class(line, *vals)

            line += subm.group().count('\n')
            pos = m.end()
            m = self.get_token(self.string, pos)

        if pos != len(self.string):
            raise Exception("Unexpected character '{}' on line {}: '{}'".format(
                self.string[pos], line + 1, self.lines[line]))

    def parse(self, cython_info=True, fname='', includes=None, disabled_tokens=()):
        self.verify_parsable_tokens()
        tokens = self.tokenize(disabled_tokens)
        rep = {'imports': [], 'messages': [], 'enums': []}
        enums = {}
        imported = {'messages': {}, 'enums': {}}
        messages = {}
        includes = includes or []
        scope = {}

        for token in tokens:
            if token.token_type == 'OPTION':
                continue

            elif token.token_type == 'SYNTAX':
                if 'proto2' == token.value:
                    self.syntax = 2
                elif 'proto3' == token.value:
                    self.syntax = 3
                    self.unsupported_tokens = ('MAP_FIELD',)
                else:
                    raise Exception("Unexpected syntax value '{}'".format(token.value))

            elif token.token_type == 'IMPORT':
                # Ignore google meta messages
                if token.value.find('google/protobuf') == 0:
                    continue

                rep['imports'].append(token.value)

                # Google's protoc only supports the use of messages and enums
                # from direct imports. So messages and enums from indirect
                # imports are not fetched here.
                imported_rep = self._parse_import(
                    token.value + '.proto',
                    fname,
                    includes,
                    disabled_tokens
                )
                imported['messages'].update(
                    (m.name, m) for m in imported_rep['messages']
                )
                imported['enums'].update(
                    (e.name, e) for e in imported_rep['enums']
                )

            elif token.token_type == 'MESSAGE':
                ret = self._parse_message(
                        token, tokens, messages, enums.copy(),
                        imported['enums']
                )

                assert token.name not in scope, (
                    "'{}' is already defined in global scope".format(
                        token.name))

                scope[token.name] = ret
                rep['messages'].append(ret)

            elif token.token_type == 'ENUM':
                rep['enums'].append(
                    self._parse_enum(token, tokens, scope)
                )
                enums[token.name] = token

            elif token.token_type == 'EXTEND':
                self._parse_extend(token, tokens)

            else:
                raise Exception("unexpected %s token on line %d: '%s'" % (
                    token.type, token.line + 1, self.lines[token.line]))

        if cython_info:
            for message in rep['messages']:
                self.add_cython_info(message)

        return rep

    @classmethod
    def parse_from_filename(cls, fname, includes, disabled_tokens=None):
        disabled_tokens = disabled_tokens or cls.unsupported_tokens
        with open(fname, 'r') as fp:
            s = fp.read()

        try:
            return cls(s).parse(fname=fname, includes=includes, disabled_tokens=disabled_tokens)
        except Exception as e:
            print('Exception while parsing {}'.format(fname))
            raise e

    def _parse_import(self, fname, parent_fname, includes, disabled_tokens):
        actual_fname = fname
        if not os.path.isabs(fname):
            for d in [os.path.dirname(parent_fname)] + includes:
                actual_fname = os.path.join(d, fname)
                if os.path.exists(actual_fname):
                    break

        return self.__class__.parse_from_filename(actual_fname, includes, disabled_tokens)

    def _process_token_enum(self, token, enums):
        """
        Helper method for processing a token using known enums.
            * replaces token.default enum string value
              by its equivalent enum int value
            * sets token.enum_default attribute to enum default string value
            * sets few other token attributes
        Args:
            token: a ParserField.
            enums: {string: ParserEnum} dictionary.
        """
        if token.default is not None:
            default = None
            enum_default = None
            found = False
            for value, entry in enums[token.type].fields.items():
                if token.default == entry.name:
                    default = value
                    enum_default = entry.full_name
                    found = True
                    break

            assert found, "Enum type '{}' has no value named '{}' on line {}: {}".format(
                token.type, token.default, token.line + 1, self.lines[token.line])
            token.default = default
            token.enum_default = enum_default

        token.enum_def = enums[token.type]
        token.enum_name = token.enum_def.full_name
        token.type = 'enum'

    def _parse_message(self, current_message, tokens, messages, enums, imported_enums):
        """
        Recursive parsing of messages.
        Args:
            current_message: the current ParserMessage object we are working on.
            tokens: a generator of Parser*.
            messages: a dictionary of all ParserMessage objects
                already known/parsed.
            enums: a dictionary of ParserEnum objects.
            imported_enums: a dictionary of ParserEnum objects.
        Returns:
            a list/hiearchy of ParserMessage objects.
        """
        token = next(tokens)
        previous = self.LBrace(-1)

        assert token.token_type == 'LBRACE', "missing opening brace on line {}: '{}'".format(
            token.line + 1, self.lines[token.line])

        for token in tokens:
            if token.token_type == 'MESSAGE':
                token.full_name = current_message.full_name + token.name
                ret = self._parse_message(token, tokens, messages, enums.copy(), imported_enums)

                assert token.name not in current_message.namespace, "'{}' is already defined in message '{}'".format(
                    token.name, current_message.name)

                current_message.messages[token.name] = ret
                current_message.namespace[token.name] = ret
                # updates the dictionary of known/parsed messages.
                messages[token.name] = current_message.messages[token.name]

            elif token.token_type == 'ENUM':
                token.full_name = current_message.full_name + token.name
                current_message.enums[token.name] = self._parse_enum(
                    token,
                    tokens,
                    current_message.namespace,
                    current_message
                )
                # updates the dictionary of known/parsed enums
                enums[token.name] = current_message.enums[token.name]

            elif token.token_type == 'ONEOF':
                current_message.oneofs[token.name] = token
                self._parse_oneof(token, tokens, current_message, messages, enums, imported_enums)

            elif token.token_type == 'MODIFIER':
                if self.syntax == 3:
                    assert token.value == 'repeated', (
                        "Illegal modifier {} on line {}: {}".format(
                            token.value, token.line + 1,
                            self.lines[token.line]))

            elif token.token_type in ('FIELD', 'MAP_FIELD'):
                self._parse_field_token(token, previous, tokens, current_message, messages, enums, imported_enums)

            elif token.token_type == 'EXTENSION':
                # Just ignore extensions for now, but don't error
                previous = token
                continue

            elif token.token_type == 'OPTION':
                # Just ignore options for now, but don't error
                previous = token
                continue

            else:
                assert token.token_type == 'RBRACE', "unexpected {} token on line {}: '{}'".format(
                    token.token_type, token.line + 1, self.lines[token.line])
                return current_message

            previous = token

        raise Exception("unexpected EOF on line {}: '{}'".format(token.line + 1, self.lines[token.line]))

    def _parse_field_token(self, token, previous, tokens, current_message, messages, enums, imported_enums):
        """Parse FIELD and MAP_FIELD token types"""
        if self.syntax == 2:
            assert previous.token_type == 'MODIFIER', "Need modifier for field on line {}: {}".format(
                token.line + 1, self.lines[token.line])
        if previous.token_type == 'MODIFIER':
            # Map fields does not accept any modifiers
            assert token.token_type == 'FIELD', "Illegal modifier '{}' for map field on line {}: {}".format(
                previous.value, token.line + 1, self.lines[token.line])

            token.modifier = previous.value
        self._parse_field(token, tokens)

        assert token.index > 0, (
            "non-positive field index on line {}: '{}'".format(token.line + 1, self.lines[token.line]))
        assert token.index not in current_message.fields, (
            "Field index {} in '{}' is already used by '{}'".format(
                token.index, current_message.name, current_message.fields[token.index].name))
        assert token.name not in current_message.namespace, (
            "'{}' is already defined in message '{}'".format(token.name, current_message.name))

        if messages.get(token.type) is not None:
            # retrieves the type "full_name"
            token.message_name = messages.get(token.type).full_name
            token.type = 'message'

        elif current_message.enums.get(token.type) is not None:
            self._process_token_enum(token, current_message.enums)

        elif enums.get(token.type) is not None:
            self._process_token_enum(token, enums)

        elif imported_enums.get(token.type) is not None:
            self._process_token_enum(token, imported_enums)

        elif (token.type not in self.scalars) and (token.type not in {'string', 'bytes'}):
            token.message_name = token.type
            token.type = 'message'

        current_message.fields[token.index] = token
        current_message.namespace[token.name] = token

    def _parse_oneof(self, oneof_token, tokens, current_message, messages, enums, imported_enums):
        token = next(tokens)
        assert token.token_type == 'LBRACE', (
            "missing opening paren on line {}: '{}'".format(
                token.line + 1, self.lines[token.line]))

        # setting previous as a place holder for the inner fields parsing
        previous = self.LBrace(-1)
        for token in tokens:
            if token.token_type == 'FIELD':
                # fields will be added to the proper message by the following
                # parser function that takes care of fields generally
                self._parse_field_token(token, previous, tokens, current_message, messages, enums, imported_enums)
                oneof_token.fields.append(token.name)
                previous = token

            else:
                assert token.token_type == 'RBRACE', "unexpected {} token on line {}: '{}'".format(
                    token.token_type, token.line + 1, self.lines[token.line])
                return oneof_token

        raise Exception("unexpected EOF on line {}: '{}'".format(
            token.line + 1, self.lines[token.line]))

    def _parse_field(self, field, tokens):
        """Parse the body of a FIELD token"""
        token = next(tokens)

        if token.token_type == 'LBRACKET':
            for token in tokens:
                if token.token_type == 'DEFAULT':
                    self._parse_default(field, tokens)
                elif token.token_type == 'PACKED':
                    field.packed = token.value
                elif token.token_type == 'DEPRECATED':
                    field.deprecated = token.value
                elif token.token_type == 'CUSTOM':
                    if self._parse_custom(field, tokens):
                        return
                elif token.token_type == 'COMMA':
                    continue
                else:
                    assert token.token_type == 'RBRACKET', "unexpected {} token on line {}: '{}'".format(
                        token.token_type, token.line + 1, self.lines[token.line])
                    return

        else:
            assert token.token_type == 'SEMICOLON', "expected ; or modifier on line {}, got {}: '{}'".format(
                token.line + 1, self.lines[token.line], token.token_type)

    def _parse_default(self, field, tokens):
        """Parse a default option"""
        token = next(tokens)

        if token.token_type == 'ENUM_FIELD':
            # This will get updated later
            field.default = token.full_name
            return
        elif token.token_type == 'NUMERIC':
            assert field.type in self.scalars, \
                "attempting to set numeric as default for non-numeric field on line {}: '{}'".format(
                    token.line + 1, self.lines[token.line])
            if field.type not in self.floats:
                assert int(token.value) == token.value, \
                    "attempting to set integer non-integer default for integer field on line {}: '{}'".format(
                        token.line + 1, self.lines[token.line])
                token.value = int(token.value)
        elif token.token_type == 'STRING':
            assert field.type in {'string', 'bytes'}, "attempting to set string as default on line {}: '{}'".format(
                token.line + 1, self.lines[token.line])
        else:
            assert token.token_type == 'BOOLEAN', "unexpected {} token on line {}: '{}'".format(
                token.token_type, token.line + 1, self.lines[token.line])

        field.default = token.value

    def _parse_custom(self, field, tokens):
        """Parse a custom option and return whether or not we hit the closing RBRACKET"""
        token = next(tokens)

        if token.token_type == 'STRING':
            field.value = token.value
            for token in tokens:
                if token.token_type == 'STRING':
                    field.value += token.value
                    continue
                elif token.token_type == 'COMMA':
                    return False
                else:
                    assert token.token_type == 'RBRACKET'
                    return True
        else:
            assert token.token_type in {'NUMERIC', 'BOOLEAN'}, "unexpected custom option value on line {}: '{}'".format(
                token.line + 1, self.lines[token.line])
            field.value = token.value
            return False

    def _parse_enum(self, current, tokens, scope, current_message=None):
        token = next(tokens)
        assert token.token_type == 'LBRACE', "missing opening brace on line {}: '{}'".format(
            token.line + 1, self.lines[token.line])

        for num, token in enumerate(tokens):
            if token.token_type == 'ENUM_FIELD':
                if num == 0:
                    if self.syntax == 3:
                        assert token.value == 0, "expected zero as first enum element on line {}, got {}: '{}'".format(
                            token.line + 1, token.value, self.lines[token.line])
                    current.default = token

                token.full_name = "{}_{}".format(current.full_name, token.name)

                assert token.name not in scope, "'{}' is already defined in {}".format(
                    token.name, "'{}'".format(current_message.name) if current_message else "global scope")

                # protoc allows value collisions with allow_alias option;
                # revisit once options are implemented
                assert token.value not in current.fields, "Enum value {} in '{}' is already used".format(
                    token.value, current.name)

                current.fields[token.value] = token
                scope[token.name] = token

                self._parse_enum_field(token, tokens)
            else:
                assert token.token_type == 'RBRACE', "unexpected {} token on line {}: '{}'".format(
                    token.token_type, token.line + 1, self.lines[token.line])
                return current

        raise Exception("unexpected EOF on line {}: '{}'".format(
            token.line + 1, self.lines[token.line]))

    def _parse_enum_field(self, field, tokens):
        token = next(tokens)

        if token.token_type == 'LBRACKET':
            for token in tokens:
                if token.token_type == 'CUSTOM':
                    if self._parse_custom(field, tokens):
                        return
                elif token.token_type == 'COMMA':
                    continue
                else:
                    assert token.token_type == 'RBRACKET', "unexpected {} token on line {}: '{}'".format(
                        token.token_type, token.line + 1, self.lines[token.line])
                    return

        else:
            assert token.token_type == 'SEMICOLON', "expected ; or modifier on line {}, got {}: '{}'".format(
                token.line + 1, token.token_type, self.lines[token.line])

    def _parse_extend(self, current, tokens):
        token = next(tokens)
        assert token.token_type == 'LBRACE', "missing opening paren on line {}: '{}'".format(
            token.line + 1, self.lines[token.line])

        # For now, just find the closing brace and return
        for token in tokens:
            if token.token_type == 'RBRACE':
                return current

    def add_cython_info(self, message):
        for index, field in message.fields.items():
            field.bitmap_idx = (index - 1) // 64
            field.bitmap_mask = 1 << ((index - 1) % 64)
            field.list_type = self.list_type_map.get(field.type, 'TypedList')
            field.fixed_width = (field.type in {
                'float', 'double', 'fixed32', 'sfixed32', 'fixed64', 'sfixed64'
            })
            field.var_width = (field.type in {
                'bool', 'enum', 'int32', 'sint32', 'uint32', 'int64', 'sint64',
                'uint64'
            })

            if field.type in self.scalars:
                field.c_type = self.c_type_map[field.type]

            if field.var_width:
                field.getter = self.getter_map[field.type]
                field.setter = self.setter_map[field.type]

        # Determine number of bitmap fields needed to record which fields have been set
        message.num_field_bitmaps = len(message.fields) // 64 + (1 if len(message.fields) % 64 > 0 else 0)

        for submessage in message.messages.values():
            self.add_cython_info(submessage)

    @classmethod
    def get_token_type_to_token_class_map(cls):
        return {
            token_class.token_type: token_class
            for token_class in cls.Token.__subclasses__()
        }

    @classmethod
    def verify_parsable_tokens(cls):
        """Raise error if any of the parsable tokens has no class defined."""
        mapping = cls.get_token_type_to_token_class_map()
        for token_type in cls.parsable_tokens:
            if token_type not in mapping:
                raise NotImplementedError("Parsable token {} has no repr class defined.".format(token_type))

    class Token(object):
        token_type = None
        line = -1
        type = None

    class Option(Token):
        token_type = 'OPTION'

        def __init__(self, line, option):
            self.line = line
            self.option = option

    class Syntax(Token):
        token_type = 'SYNTAX'

        def __init__(self, line, value):
            self.line = line
            self.value = value

    class Import(Token):
        token_type = 'IMPORT'

        def __init__(self, line, value):
            self.line = line
            self.value = value

    class Message(Token):
        token_type = 'MESSAGE'

        def __init__(self, line, name):
            self.line = line
            self.name = name
            # full_name may later be overridden with parent hierarchy
            self.full_name = name
            self.messages = {}
            self.enums = {}
            self.oneofs = {}
            self.fields = {}
            self.namespace = {}
            self.num_field_bitmaps = 0

    class Modifier(Token):
        token_type = 'MODIFIER'

        def __init__(self, line, value):
            self.line = line
            self.value = value

    class Field(Token):
        token_type = 'FIELD'

        def __init__(self, line, ftype, name, index):
            self.line = line
            self.modifier = None
            self.type = ftype
            self.name = name
            self.index = int(index)
            self.default = None
            self.packed = False
            self.deprecated = False

        def get_key(self):
            if self.modifier == 'repeated' and self.packed:
                return (self.index << 3) | 2
            elif self.type in {'message', 'string', 'bytes'}:
                return (self.index << 3) | 2
            elif self.type in {'fixed64', 'sfixed64', 'double'}:
                return (self.index << 3) | 1
            elif self.type in {'fixed32', 'sfixed32', 'float'}:
                return (self.index << 3) | 5
            else:
                return (self.index << 3) | 0

    class MapField(Token):
        token_type = 'MAP_FIELD'

        def __init__(self, line, key_type, value_type, name, index):
            self.line = line
            # value type is set to self.type if it is processed similarly
            # to the Field token.
            self.type = value_type
            self.key_type = key_type
            self.name = name
            self.index = int(index)

            # currently copied from the Field token so they are processed the
            # same way. Maybe it is not needed if these are not supported for
            # map fields
            self.default = None
            self.packed = False
            self.deprecated = False

            # Map fields does not support modifiers both for syntax 2 and 3
            # self.modifier = None

        def get_key(self):
            raise NotImplementedError(
                "MapField implemented currently only for parsing purposes."
            )

    class LBracket(Token):
        token_type = 'LBRACKET'

        def __init__(self, line):
            self.line = line

    class RBracket(Token):
        token_type = 'RBRACKET'

        def __init__(self, line):
            self.line = line

    class Default(Token):
        token_type = 'DEFAULT'

        def __init__(self, line):
            self.line = line

    class Packed(Token):
        token_type = 'PACKED'

        def __init__(self, line, value):
            self.line = line
            self.value = value == 'true'

    class Deprecated(Token):
        token_type = 'DEPRECATED'

        def __init__(self, line, value):
            self.line = line
            self.value = value == 'true'

    class Custom(Token):
        token_type = 'CUSTOM'

        def __init__(self, line, name):
            self.line = line
            self.name = name
            self.value = None

    class Comma(Token):
        token_type = 'COMMA'

        def __init__(self, line):
            self.line = line

    class Semicolon(Token):
        token_type = 'SEMICOLON'

        def __init__(self, line):
            self.line = line

    class Enum(Token):
        token_type = 'ENUM'

        def __init__(self, line, name):
            self.line = line
            self.name = name
            self.fields = {}
            self.default = None
            # full_name may later be overridden with parent hierarchy
            self.full_name = name

    class EnumField(Token):
        token_type = 'ENUM_FIELD'

        def __init__(self, line, name, value=None):
            self.line = line
            self.name = name
            if value is not None:
                self.value = int(value, 0)
            else:
                self.value = None
            # full_name may later be overridden with parent hierarchy
            self.full_name = name

    class Numeric(Token):
        token_type = 'NUMERIC'

        def __init__(self, line, value):
            self.line = line
            self.value = float(value)

    class String(Token):
        token_type = 'STRING'

        def __init__(self, line, value):
            self.line = line
            self.value = value

    class Boolean(Token):
        token_type = 'BOOLEAN'

        def __init__(self, line, value):
            self.line = line
            self.value = True if value == 'true' else False

    class LBrace(Token):
        token_type = 'LBRACE'

        def __init__(self, line):
            self.line = line

    class RBrace(Token):
        token_type = 'RBRACE'

        def __init__(self, line):
            self.line = line

    class Extension(Token):
        token_type = 'EXTENSION'

        def __init__(self, line, low, hi):
            self.line = line
            self.low = low
            self.hi = hi

    class Oneof(Token):
        token_type = 'ONEOF'

        def __init__(self, line, name):
            self.line = line
            self.name = name
            self.fields = []

    class Extend(Token):
        token_type = 'EXTEND'

        def __init__(self, line, name):
            self.line = line
            self.name = name


class Proto3Parser(Parser):
    syntax = 3
    unsupported_tokens = ('MAP_FIELD',)



"""Setuptools integration."""
import os
import sys

from pyrobuf.compile import Compiler

if sys.version_info.major == 3:
    basestring = str
    _FileExistsError = FileExistsError
else:
    _FileExistsError = OSError


def add_pyrobuf_module(dist, pyrobuf_module):
    dir_name = "pyrobuf/_" + pyrobuf_module
    package = "{}_{}".format(dist.get_name(), pyrobuf_module)
    try:
        os.makedirs(os.path.join(dir_name, package))
    except _FileExistsError:
        pass
    compiler = Compiler([pyrobuf_module], out=dir_name, package=package)
    compiler.extend(dist)


def pyrobuf_modules(dist, attr, value):
    assert attr == 'pyrobuf_modules'
    if isinstance(value, basestring):
        value = [value]

    for pyrobuf_module in value:
        add_pyrobuf_module(dist, pyrobuf_module)



import unittest

Test = None
TestRef = None


class MergeFromTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global Test, TestRef
        from test_message_proto import Test
        from test_ref_message_proto import TestRef

    def test_extend_with_generic_list(self):
        t = Test()
        refs = [TestRef(), TestRef(), TestRef()]

        for i in range(3):
            refs[i].field1 = i

        t.list_ref.extend(refs)

        for i in range(3):
            self.assertEqual(t.list_ref[i].field1, i)
            self.assertIsNot(t.list_ref[i], refs[i])

    def test_set_list_with_keyword_arg(self):
        refs = [TestRef(), TestRef(), TestRef()]

        for i in range(3):
            refs[i].field1 = i

        t = Test(list_ref=refs)

        for i in range(3):
            self.assertEqual(t.list_ref[i].field1, i)
            self.assertIsNot(t.list_ref[i], refs[i])



def test_before_overflow():
    from issue_11_proto import A
    a = A()
    a.a0 = 0x7FFFFFFF
    assert A.FromString(a.SerializeToString()).a0 == 2147483647


def test_after_overflow():
    from issue_11_proto import A
    a = A()
    a.a0 = 0x80000000
    assert A.FromString(a.SerializeToString()).a0 == 2147483648



import unittest

from create_message import create_an_test


class ItemsTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global Test
        from test_message_proto import Test

    def test_fields(self):
        test = create_an_test()
        fields = [
            'timestamp',
            'field',
            'string_field',
            'list_fieldx',
            'substruct',
            'test_ref',
            'enum_field',
            'list_ref',
            'another_substruct',
            'req_field',
            'negative_32'
        ]
        self.assertEqual(fields, list(test.Fields()))

    def test_items(self):
        test = create_an_test()
        for field, value in test.Items():
            self.assertEqual(value, getattr(test, field))

    def test_setters(self):
        source = create_an_test()
        dest = Test()

        for setter, value in zip(dest.Setters(), source.Values()):
            setter(value)

        for source_val, dest_val in zip(source.Values(), dest.Values()):
            self.assertEqual(source_val, dest_val)



import unittest

TestOneof = None
SubMsg = None


class TestOneofTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global TestOneof, SubMsg
        from test_oneof_3_proto import TestOneof, SubMsg

    def test_one_of(self):
        test = TestOneof()
        self.assertFalse(test.HasField('oneof_int64'))
        self.assertFalse(test.HasField('oneof_string'))
        self.assertFalse(test.HasField('oneof_float'))
        self.assertFalse(test.HasField('oneof_submsg'))
        self.assertEqual(0, test.oneof_int64)
        self.assertEqual(0.0, test.oneof_float)
        self.assertEqual('', test.oneof_string)

        test.oneof_int64 = 42
        self.assertTrue(test.HasField('oneof_int64'))
        self.assertEqual(test.oneof_int64, 42)
        self.assertFalse(test.HasField('oneof_string'))
        self.assertFalse(test.HasField('oneof_float'))
        self.assertFalse(test.HasField('oneof_submsg'))
        self.assertEqual(0.0, test.oneof_float)
        self.assertEqual('', test.oneof_string)

        test.oneof_string = 'the string'
        self.assertTrue(test.HasField('oneof_string'))
        self.assertEqual(test.oneof_string, 'the string')
        self.assertFalse(test.HasField('oneof_int64'))
        self.assertFalse(test.HasField('oneof_float'))
        self.assertFalse(test.HasField('oneof_submsg'))
        self.assertEqual(0, test.oneof_int64)
        self.assertEqual(0.0, test.oneof_float)

        test.oneof_float = 42.0
        self.assertTrue(test.HasField('oneof_float'))
        self.assertEqual(test.oneof_float, 42.0)
        self.assertFalse(test.HasField('oneof_int64'))
        self.assertFalse(test.HasField('oneof_string'))
        self.assertFalse(test.HasField('oneof_submsg'))
        self.assertEqual(0, test.oneof_int64)
        self.assertEqual('', test.oneof_string)

        test.oneof_submsg.some_field = 42
        self.assertTrue(test.HasField('oneof_submsg'))
        self.assertTrue(isinstance(test.oneof_submsg, SubMsg))
        self.assertFalse(test.HasField('oneof_int64'))
        self.assertFalse(test.HasField('oneof_float'))
        self.assertFalse(test.HasField('oneof_string'))
        self.assertEqual(0, test.oneof_int64)
        self.assertEqual(0.0, test.oneof_float)
        self.assertEqual('', test.oneof_string)

    def test_oneof_clears(self):
        test = TestOneof()

        test.oneof_int64 = 42
        test.ClearField('oneof_int64')
        self.assertFalse(test.HasField('oneof_int64'))
        self.assertEqual(0, test.oneof_int64)

        test.oneof_submsg.some_field = 42
        self.assertTrue(test.HasField('oneof_submsg'))
        test.ClearField('oneof_submsg')
        self.assertFalse(test.HasField('oneof_submsg'))



import unittest

TestCustomOptions = None


class TestCustomOptionsTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global TestCustomOptions
        from test_custom_options_proto import TestCustomOptions

    def test_fields(self):
        test = TestCustomOptions()
        self.assertEqual("bar", test.field2)
        self.assertEqual(1, test.field3)



import sys


if sys.version_info.major == 2:
    import messages.test_message_pb2 as google_test


def create_an_test(protover=2):
    # print LIB
    if protover == 2:
        import test_message_proto as an_test
    elif protover == 3:
        import test_message_3_proto as an_test
    test = an_test.Test()
    test.timestamp = 539395200
    test.field = 10689
    test.string_field = "go goats!"

    for i in range(5):
        test.list_fieldx.append(i * 100)

    test.substruct.field1 = 12345
    test.substruct.field2 = "hello"
    test.substruct.field3.field1 = 1419.67
    test.substruct.field3.ss2_field2 = "goodbye"
    test.substruct.list.append(354.94)

    obj = test.substruct.list_object.add()
    obj.field1 = 3.14159
    obj.ss2_field2 = "pi"

    test.substruct.list_string.append("something")

    test.test_ref.timestamp = 539395200
    test.test_ref.field1 = 1111
    test.test_ref.field2 = 1.2345
    test.test_ref.field3 = "foo"

    obj = test.list_ref.add()
    obj.timestamp = 539395200
    obj.field1 = 1111
    obj.field2 = 1.2345
    obj.field3 = "foo"

    test.another_substruct.string_field = "what's up?"
    test.another_substruct.fixed_string_field = "nothing much"
    test.another_substruct.int_field = 24
    test.another_substruct.another_int_field = 87

    test.another_substruct.substruct_ref.timestamp = 539395200
    test.another_substruct.substruct_ref.field1 = 1111
    test.another_substruct.substruct_ref.field2 = 1.2345
    test.another_substruct.substruct_ref.field3 = "foo"

    test.req_field = -80914
    test.negative_32 = -1

    return test


if sys.version_info.major == 2:
    def create_google_test():
        test = google_test.Test()
        test.timestamp = 539395200
        test.field = 10689
        test.string_field = "go goats!"

        for i in range(5):
            test.list_fieldx.append(i * 100)

        test.substruct.field1 = 12345
        test.substruct.field2 = "hello"
        test.substruct.field3.field1 = 1419.67
        test.substruct.field3.ss2_field2 = "goodbye"
        test.substruct.list.append(354.94)

        obj = test.substruct.list_object.add()
        obj.field1 = 3.14159
        obj.ss2_field2 = "pi"

        test.substruct.list_string.append("something")

        test.test_ref.timestamp = 539395200
        test.test_ref.field1 = 1111
        test.test_ref.field2 = 1.2345
        test.test_ref.field3 = "foo"

        obj = test.list_ref.add()
        obj.timestamp = 539395200
        obj.field1 = 1111
        obj.field2 = 1.2345
        obj.field3 = "foo"

        test.another_substruct.string_field = "what's up?"
        test.another_substruct.fixed_string_field = "nothing much"
        test.another_substruct.int_field = 24
        test.another_substruct.another_int_field = 87

        test.another_substruct.substruct_ref.timestamp = 539395200
        test.another_substruct.substruct_ref.field1 = 1111
        test.another_substruct.substruct_ref.field2 = 1.2345
        test.another_substruct.substruct_ref.field3 = "foo"

        test.req_field = -80914
        test.negative_32 = -1

        return test

    def create_buffer():
        test = create_google_test()
        return test.SerializeToString()



import unittest

EmptyMessageWithNoFields = None


class EmptyMessageTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global EmptyMessageWithNoFields
        from test_message_field_types_proto import EmptyMessageWithNoFields

    def setUp(self):
        self.message = EmptyMessageWithNoFields()

    def test_message_with_no_fields_parse_from_string(self):
        self.assertEqual(0, self.message.ParseFromString(b""))

    def test_message_with_no_fields_serialize_to_string(self):
        self.assertEqual(b"", self.message.SerializeToString())

    def test_message_with_no_fields_serialize_to_json(self):
        self.assertEqual("{}", self.message.SerializeToJson())

    def test_message_with_no_fields_serialize_to_dict(self):
        self.assertEqual({}, self.message.SerializeToDict())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: test_message.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)


import test_ref_message_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='test_message.proto',
  package='',
  serialized_pb='\n\x12test_message.proto\x1a\x16test_ref_message.proto\"\xf9\x05\n\x04Test\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\r\n\x05\x66ield\x18\x02 \x01(\x03\x12\x14\n\x0cstring_field\x18\x03 \x01(\t\x12\x13\n\x0blist_fieldx\x18\x04 \x03(\x03\x12\x1c\n\tsubstruct\x18\x05 \x01(\x0b\x32\t.Test.Ss1\x12\x1a\n\x08test_ref\x18\x06 \x01(\x0b\x32\x08.TestRef\x12\x36\n\nenum_field\x18\x07 \x01(\x0e\x32\x0f.Test.EnumField:\x11TEST_ENUM_FIELD_0\x12\x1a\n\x08list_ref\x18\x08 \x03(\x0b\x32\x08.TestRef\x12$\n\x11\x61nother_substruct\x18\t \x01(\x0b\x32\t.Test.Ss3\x12\x11\n\treq_field\x18\n \x02(\x03\x12\x13\n\x0bnegative_32\x18\x0b \x01(\x05\x1a\xec\x01\n\x03Ss1\x12\x15\n\x06\x66ield1\x18\x01 \x01(\x03:\x05\x31\x32\x33\x34\x35\x12\x0e\n\x06\x66ield2\x18\x02 \x01(\t\x12\x1d\n\x06\x66ield3\x18\x03 \x01(\x0b\x32\r.Test.Ss1.Ss2\x12\x0c\n\x04list\x18\x04 \x03(\x01\x12$\n\x0blist_object\x18\x05 \x03(\x0b\x32\x0f.Test.Ss1.Thing\x12\x13\n\x0blist_string\x18\x06 \x03(\t\x1a)\n\x03Ss2\x12\x0e\n\x06\x66ield1\x18\x01 \x01(\x01\x12\x12\n\nss2_field2\x18\x02 \x01(\t\x1a+\n\x05Thing\x12\x0e\n\x06\x66ield1\x18\x01 \x01(\x01\x12\x12\n\nss2_field2\x18\x02 \x01(\t\x1a\x86\x01\n\x03Ss3\x12\x14\n\x0cstring_field\x18\x01 \x01(\t\x12\x1a\n\x12\x66ixed_string_field\x18\x02 \x01(\t\x12\x11\n\tint_field\x18\x03 \x01(\r\x12\x19\n\x11\x61nother_int_field\x18\x04 \x01(\r\x12\x1f\n\rsubstruct_ref\x18\x05 \x01(\x0b\x32\x08.TestRef\"P\n\tEnumField\x12\x15\n\x11TEST_ENUM_FIELD_0\x10\x00\x12\x15\n\x11TEST_ENUM_FIELD_1\x10\x01\x12\x15\n\x11TEST_ENUM_FIELD_2\x10\x02')



_TEST_ENUMFIELD = _descriptor.EnumDescriptor(
  name='EnumField',
  full_name='Test.EnumField',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TEST_ENUM_FIELD_0', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEST_ENUM_FIELD_1', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEST_ENUM_FIELD_2', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=728,
  serialized_end=808,
)


_TEST_SS1_SS2 = _descriptor.Descriptor(
  name='Ss2',
  full_name='Test.Ss1.Ss2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='field1', full_name='Test.Ss1.Ss2.field1', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ss2_field2', full_name='Test.Ss1.Ss2.ss2_field2', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=503,
  serialized_end=544,
)

_TEST_SS1_THING = _descriptor.Descriptor(
  name='Thing',
  full_name='Test.Ss1.Thing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='field1', full_name='Test.Ss1.Thing.field1', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ss2_field2', full_name='Test.Ss1.Thing.ss2_field2', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=546,
  serialized_end=589,
)

_TEST_SS1 = _descriptor.Descriptor(
  name='Ss1',
  full_name='Test.Ss1',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='field1', full_name='Test.Ss1.field1', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=12345,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field2', full_name='Test.Ss1.field2', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field3', full_name='Test.Ss1.field3', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list', full_name='Test.Ss1.list', index=3,
      number=4, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_object', full_name='Test.Ss1.list_object', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_string', full_name='Test.Ss1.list_string', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TEST_SS1_SS2, _TEST_SS1_THING, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=353,
  serialized_end=589,
)

_TEST_SS3 = _descriptor.Descriptor(
  name='Ss3',
  full_name='Test.Ss3',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_field', full_name='Test.Ss3.string_field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='fixed_string_field', full_name='Test.Ss3.fixed_string_field', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='int_field', full_name='Test.Ss3.int_field', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='another_int_field', full_name='Test.Ss3.another_int_field', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='substruct_ref', full_name='Test.Ss3.substruct_ref', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=592,
  serialized_end=726,
)

_TEST = _descriptor.Descriptor(
  name='Test',
  full_name='Test',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='Test.timestamp', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field', full_name='Test.field', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='string_field', full_name='Test.string_field', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_fieldx', full_name='Test.list_fieldx', index=3,
      number=4, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='substruct', full_name='Test.substruct', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_ref', full_name='Test.test_ref', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enum_field', full_name='Test.enum_field', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_ref', full_name='Test.list_ref', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='another_substruct', full_name='Test.another_substruct', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='req_field', full_name='Test.req_field', index=9,
      number=10, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='negative_32', full_name='Test.negative_32', index=10,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TEST_SS1, _TEST_SS3, ],
  enum_types=[
    _TEST_ENUMFIELD,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=47,
  serialized_end=808,
)

_TEST_SS1_SS2.containing_type = _TEST_SS1;
_TEST_SS1_THING.containing_type = _TEST_SS1;
_TEST_SS1.fields_by_name['field3'].message_type = _TEST_SS1_SS2
_TEST_SS1.fields_by_name['list_object'].message_type = _TEST_SS1_THING
_TEST_SS1.containing_type = _TEST;
_TEST_SS3.fields_by_name['substruct_ref'].message_type = test_ref_message_pb2._TESTREF
_TEST_SS3.containing_type = _TEST;
_TEST.fields_by_name['substruct'].message_type = _TEST_SS1
_TEST.fields_by_name['test_ref'].message_type = test_ref_message_pb2._TESTREF
_TEST.fields_by_name['enum_field'].enum_type = _TEST_ENUMFIELD
_TEST.fields_by_name['list_ref'].message_type = test_ref_message_pb2._TESTREF
_TEST.fields_by_name['another_substruct'].message_type = _TEST_SS3
_TEST_ENUMFIELD.containing_type = _TEST;
DESCRIPTOR.message_types_by_name['Test'] = _TEST

class Test(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Ss1(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType

    class Ss2(_message.Message):
      __metaclass__ = _reflection.GeneratedProtocolMessageType
      DESCRIPTOR = _TEST_SS1_SS2

      # @@protoc_insertion_point(class_scope:Test.Ss1.Ss2)

    class Thing(_message.Message):
      __metaclass__ = _reflection.GeneratedProtocolMessageType
      DESCRIPTOR = _TEST_SS1_THING

      # @@protoc_insertion_point(class_scope:Test.Ss1.Thing)
    DESCRIPTOR = _TEST_SS1

    # @@protoc_insertion_point(class_scope:Test.Ss1)

  class Ss3(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _TEST_SS3

    # @@protoc_insertion_point(class_scope:Test.Ss3)
  DESCRIPTOR = _TEST

  # @@protoc_insertion_point(class_scope:Test)


# @@protoc_insertion_point(module_scope)



import unittest

from create_message import create_an_test


class CachingTest(unittest.TestCase):

    def test_basic_no_difference(self):
        test = create_an_test()
        self.assertEqual(test.SerializeToString(),
                         test.SerializeToString(cache=True))

    def test_basic_caching(self):
        test = create_an_test()
        serialized = test.SerializeToString()
        print(serialized)
        test.ParseFromString(serialized, cache=True)
        self.assertEqual(serialized, test.SerializeToString(cache=True))
        self.assertEqual(serialized, test.SerializeToString(cache=True))

    def test_update(self):
        test = create_an_test()
        serialized = test.SerializeToString(cache=True)
        test.field = test.field + 1
        self.assertNotEqual(serialized, test.SerializeToString(cache=True))
        test.ParseFromString(serialized, cache=True)
        test.field = test.field + 1
        self.assertNotEqual(serialized, test.SerializeToString(cache=True))

    def test_update_child(self):
        test = create_an_test()
        serialized = test.SerializeToString(cache=True)
        test.substruct.field1 = test.substruct.field1 + 1
        self.assertNotEqual(serialized, test.SerializeToString(cache=True))
        test.ParseFromString(serialized, cache=True)
        test.substruct.field1 = test.substruct.field1 + 1
        self.assertNotEqual(serialized, test.SerializeToString(cache=True))



import unittest

M = None
MN = None
MN2 = None


class NestedTest(unittest.TestCase):
    """
    Verifies we can create instance from nested proto file.
    https://github.com/appnexus/pyrobuf/issues/55
    """

    @classmethod
    def setUpClass(cls):
        global M
        global MN
        global MN2
        from test_nested_issue55_proto import M, MN, MN2

    def test_use_nested(self):
        """
        Simple test that just verifies we can creates M, N and N2 objects.
        """
        message_m = M()
        message_n = MN()
        message_n2 = MN2()


if __name__ == "__main__":
    unittest.main()




import os
import sys

from distutils.util import get_platform
from pyrobuf.compile import Compiler


def pytest_sessionstart(session):
    # Build test messages from proto specs
    here = os.path.dirname(os.path.abspath(__file__))
    proto = [os.path.join(here, 'proto', filename)
             for filename in os.listdir(os.path.join(here, 'proto'))]
    compiler = Compiler(proto, out='tests/out', build='tests/build')
    compiler.compile()

    proto3 = [os.path.join(here, 'proto3', filename)
             for filename in os.listdir(os.path.join(here, 'proto3'))]
    compiler3 = Compiler(proto3, proto3=True, out='tests/out', build='tests/build')
    compiler3.compile()

    # Add test directory into path
    if here not in sys.path:
        sys.path.append(here)

    # Insert built messages into path
    build = os.path.join(here, 'build')
    lib_path = os.path.join(build, "lib.{0}-{1}".format(get_platform(),
                                                        sys.version[0:3]))

    if lib_path not in sys.path:
        sys.path.insert(0, lib_path)



import unittest


TestIsInitialized = None
SubMessage = None
TestWithRequiredSubMessage = None


class MergeFromTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        global TestIsInitialized, SubMessage, TestWithRequiredSubMessage
        from test_is_initialized_proto import TestIsInitialized, SubMessage, TestWithRequiredSubMessage

    def test_new_message_is_not_initialized(self):
        message = TestIsInitialized()
        self.assertFalse(message.IsInitialized())

    def test_message_with_all_required_fields_set_is_initialized(self):
        message = TestIsInitialized()
        message.req_field = 1
        self.assertTrue(message.IsInitialized())

    def test_new_message_with_required_submessage_is_not_initialized(self):
        message = TestWithRequiredSubMessage()
        self.assertFalse(message.IsInitialized())

    def test_message_with_initialized_required_submessage_is_initialized(self):
        message = TestWithRequiredSubMessage()
        message.req_sub.req_field = 1
        self.assertTrue(message.IsInitialized())

    def test_message_with_uninitialized_submessage_is_not_initialized(self):
        message = TestIsInitialized()
        message.req_field = 1
        message.sub_message.opt_field = 2
        self.assertFalse(message.IsInitialized())

    def test_message_with_initialized_submessage_is_initialized(self):
        message = TestIsInitialized()
        message.req_field = 1
        message.sub_message.req_field = 2
        self.assertTrue(message.IsInitialized())

    def test_message_with_uninitialized_repeated_submessage_is_not_initialized(self):
        message = TestIsInitialized()
        message.req_field = 1
        message.list_sub.add().opt_field = 2
        self.assertFalse(message.IsInitialized())

    def test_message_with_initialized_repeated_submessage_is_initialized(self):
        message = TestIsInitialized()
        message.req_field = 1
        message.list_sub.add().req_field = 2
        self.assertTrue(message.IsInitialized())
